"""
 Application for testing syncing algorithm

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

import sys
import os
import time
import shutil
import unittest
import xmlrunner
import subprocess
import re
from sync_test_app import SyncTestApp
from sync_test import SyncTest
import logging
import argparse


class SyncTestMegaCliApp(SyncTestApp):
    """
    operates with megacli application
    """
    def __init__(self, local_mount_in, local_mount_out, delete_tmp_files=True, use_large_files=True, check_if_alive=True):
        """
        local_mount_in: local upsync folder
        local_mount_out: local downsync folder
        """
        self.work_dir = os.path.join(".", "work_dir")
        SyncTestApp.__init__(self, local_mount_in, local_mount_out, self.work_dir, delete_tmp_files, use_large_files)
        self.check_if_alive = check_if_alive

    def sync(self):
        time.sleep(5)

    def start(self):
        # try to create work dir
        return True

    def finish(self):
        try:
            shutil.rmtree(self.work_dir)
        except OSError, e:
            logging.error("Failed to remove dir: %s (%s)" % (self.work_dir, e))

    def is_alive(self):
        """
        return True if application instance is running
        """
        if not self.check_if_alive:
            return True

        s = subprocess.Popen(["ps", "axw"], stdout=subprocess.PIPE)
        for x in s.stdout:
            if re.search("megacli", x):
                return True
        return False

    def pause(self):
        """
        pause application
        """
        # TODO: implement this !
        raise NotImplementedError("Not Implemented !")

    def unpause(self):
        """
        unpause application
        """
        # TODO: implement this !
        raise NotImplementedError("Not Implemented !")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--test1", help="test_create_delete_files", action="store_true")
    parser.add_argument("--test2", help="test_create_rename_delete_files", action="store_true")
    parser.add_argument("--test3", help="test_create_delete_dirs", action="store_true")
    parser.add_argument("--test4", help="test_create_rename_delete_dirs", action="store_true")
    parser.add_argument("--test5", help="test_sync_files_write", action="store_true")
    parser.add_argument("--test6", help="test_local_operations", action="store_true")
    parser.add_argument("--test7", help="test_update_mtime", action="store_true")
    parser.add_argument("--test8", help="test_create_rename_delete_unicode_files_dirs", action="store_true")
    parser.add_argument("--test9", help="test_create_move_delete_files", action="store_true")
    parser.add_argument("--test10", help="test_mimic_update_with_backup_files", action="store_true")
    parser.add_argument("-a", "--all", help="run all tests", action="store_true")
    parser.add_argument("-b", "--basic", help="run basic, stable tests", action="store_true")
    parser.add_argument("-d", "--debug", help="use debug output", action="store_true")
    parser.add_argument("-l", "--large", help="use large files for testing", action="store_true")
    parser.add_argument("-n", "--nodelete", help="Do not delete work files", action="store_false")
    parser.add_argument("-c", "--check", help="Do not check if megacli is running (useful, if other application is used for testing)", action="store_false")
    parser.add_argument("upsync_dir", help="local upsync directory")
    parser.add_argument("downsync_dir", help="local downsync directory")
    args = parser.parse_args()

    if args.debug:
        lvl = logging.DEBUG
    else:
        lvl = logging.INFO

    if args.all:
        args.test1 = args.test2 = args.test3 = args.test4 = args.test5 = args.test6 = args.test7 = args.test8 = args.test9 = args.test10 = True
    if args.basic:
        args.test1 = args.test2 = args.test3 = args.test4 = True

    logging.StreamHandler(sys.stdout)
    logging.basicConfig(format='[%(asctime)s] %(message)s', datefmt='%Y-%m-%d %H:%M:%S', level=lvl)

    logging.info("")
    logging.info("1) Start the first [megacli] and run the following command:  sync " + args.upsync_dir + " [remote folder]")
    logging.info("2) Start the second [megacli] and run the following command:  sync " + args.downsync_dir + " [remote folder]")
    logging.info("3) Wait for both folders get fully synced")
    logging.info("4) Run: python %s", sys.argv[0])
    logging.info("")
    logging.info("   Make sure you have unittest module installed:   pip install unittest-xml-reporting")
    logging.info("")
    time.sleep(5)

    with SyncTestMegaCliApp(args.upsync_dir, args.downsync_dir, args.nodelete, args.large, args.check) as app:
        suite = unittest.TestSuite()

        if args.test1:
            suite.addTest(SyncTest("test_create_delete_files", app))

        if args.test2:
            suite.addTest(SyncTest("test_create_rename_delete_files", app))

        if args.test3:
            suite.addTest(SyncTest("test_create_delete_dirs", app, ))

        if args.test4:
            suite.addTest(SyncTest("test_create_rename_delete_dirs", app))

        if args.test5:
            suite.addTest(SyncTest("test_sync_files_write", app))

        if args.test6:
            suite.addTest(SyncTest("test_local_operations", app))

        if args.test7:
            suite.addTest(SyncTest("test_update_mtime", app))

        if args.test8:
            suite.addTest(SyncTest("test_create_rename_delete_unicode_files_dirs", app))

        if args.test9:
            suite.addTest(SyncTest("test_create_move_delete_files", app))
            
        if args.test10:
            suite.addTest(SyncTest("test_mimic_update_with_backup_files", app))

        testRunner = xmlrunner.XMLTestRunner(output='test-reports')
        testRunner.run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Simple CRUD example for the Python bindings."""

## Created: 16 May 2015 Guy Kloss <gk@mega.co.nz>
##
## (c) 2015 by Mega Limited, Auckland, New Zealand
##     https://mega.nz/
##     Simplified (2-clause) BSD License.
##
## You should have received a copy of the license along with this
## program.
##
## This file is part of the Mega SDK Python bindings example code.
##
## This code is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

__author__ = 'Guy Kloss <gk@mega.co.nz>'

import sys
import os
import logging
import time
import threading
import json
import getpass

from mega import (MegaApi, MegaListener, MegaError, MegaRequest, MegaNode)

# Mega SDK application key.
# Generate one for free here: https://mega.nz/#sdk
APP_KEY = 'ox8xnQZL'

# Credentials file to look for login info.
CREDENTIALS_FILE = 'credentials.json'


class AppListener(MegaListener):
    """
    A listener that will contain callback methods called upon events
    from the asyncrhonous API. Only a certain number of usefuul
    callbacks are implemented.
    """
    
    # Inhibit sending an event on callbacks from these requests.
    _NO_EVENT_ON = (MegaRequest.TYPE_LOGIN,
                    MegaRequest.TYPE_FETCH_NODES)

    def __init__(self, continue_event):
        """
        Constructor.

        :param continue_event: Event to use for notifying our main code
            upon completion of the asyncrhonous operation.
        """
        self.continue_event = continue_event
        self.root_node = None
        super(AppListener, self).__init__()


    def onRequestStart(self, api, request):
        """
        Called upon starting the async API operation.

        :param api: Reference to the API object.
        :param request: Reference to the request this operation belongs to.
        """
        logging.info('Request start ({})'.format(request))


    def onRequestFinish(self, api, request, error):
        """
        Called upon finishing the async API operation.

        :param api: Reference to the API object.
        :param request: Reference to the request this operation belongs to.
        :param eror: Error information. API_OK if it finishes without a
            problem.
        """
        logging.info('Request finished ({}); Result: {}'
                     .format(request, error))

        request_type = request.getType()
        if request_type == MegaRequest.TYPE_LOGIN:
            api.fetchNodes()
        elif request_type == MegaRequest.TYPE_FETCH_NODES:
            self.root_node = api.getRootNode()
        elif request_type == MegaRequest.TYPE_ACCOUNT_DETAILS:
            account_details = request.getMegaAccountDetails()
            logging.info('Account details received')
            logging.info('Storage: {} of {} ({} %)'
                         .format(account_details.getStorageUsed(),
                                 account_details.getStorageMax(),
                                 100 * account_details.getStorageUsed()
                                 / account_details.getStorageMax()))
            logging.info('Pro level: {}'.format(account_details.getProLevel()))

        # Notify other thread to go on.
        if request_type not in self._NO_EVENT_ON:
            self.continue_event.set()


    def onRequestTemporaryError(self, api, request, error):
        """
        Called upon a temprary error of the API operation. To be continued
        (involving further callbacks).

        :param api: Reference to the API object.
        :param request: Reference to the request this operation belongs to.
        :param eror: Error information.
        """
        logging.info('Request temporary error ({}); Error: {}'
                     .format(request, error))


    def onTransferFinish(self, api, transfer, error):
        """
        Called upon finishing the async API transfer operation.

        :param api: Reference to the API object.
        :param request: Reference to the request this operation belongs to.
        :param eror: Error information. API_OK if it finishes without a
            problem.
        """
        logging.info('Transfer finished ({}); Result: {}'
                     .format(transfer, transfer.getFileName(), error))
        self.continue_event.set()


    def onTransferUpdate(self, api, transfer):
        """
        Called intermittently to provide updates on the async API transfer
        operation.

        :param api: Reference to the API object.
        :param transfer: Information about the transfer.
        """
        logging.info('Transfer update ({} {});'
                     ' Progress: {} KB of {} KB, {} KB/s'
                     .format(transfer,
                             transfer.getFileName(),
                             transfer.getTransferredBytes() / 1024,
                             transfer.getTotalBytes() / 1024,
                             transfer.getSpeed() / 1024))


    def onTransferTemporaryError(self, api, transfer, error):
        """
        Called upon a temprary error of the API transfer operation. To be
        continued (involving further callbacks).

        :param api: Reference to the API object.
        :param transfer: Information about the transfer.
        :param eror: Error information.
        """
        logging.info('Transfer temporary error ({} {}); Error: {}'
                     .format(transfer, transfer.getFileName(), error))


    def onUsersUpdate(self, api, users):
        """
        Called upon an update from the API to a user's contacts.

        :param api: Reference to the API object.
        :param users: List that contains the new or updated contacts.
        """
        if users != None:
            logging.info('Users updated ({})'.format(users.size()))


    def onNodesUpdate(self, api, nodes):
        """
        Called upon an update from the API for changed/updated storage nodes.

        :param api: Reference to the API object.
        :param nodes: List that contains the new or updated nodes.
        """
        if nodes != None:
            logging.info('Nodes updated ({})'.format(nodes.size()))
        self.continue_event.set()


class AsyncExecutor(object):
    """
    Simple helper that will wrap asynchronous invocations of API operations
    to notify the caller via an event upon completion.

    This executor is "simple", it is not suitable for overlapping executions,
    but only for a sequential chaining of API calls.
    """
    
    def __init__(self):
        """
        Constructor that creates an event used for notification from the
        API thread.
        """
        self.continue_event = threading.Event()

    def do(self, function, args):
        """
        Performs the asynchronous operation and waits for a signal from the
        Mega API using the event.

        :param function: Callable Mega API operation.
        :param args: Arguments to the callable.
        """
        self.continue_event.clear()
        function(*args)
        self.continue_event.wait()


def worker(api, listener, executor, credentials):
    """
    Sequential collection of (CRUD) operations on the Mega API.

    :param api: Reference to the Mega API object.
    :param listener: Listener to receie callbacks and results from the
        Mega API.
    :param executor: Simple asynchronous API command executor to enable
        a sequential/synchronous flow of operations in this function.
    :param credentials: Dictionary containing a `user` and a `password`
        string to log into the Mega account.
    """
    # Log in.
    logging.info('*** start: login ***')
    executor.do(api.login, (str(credentials['user']),
                            str(credentials['password'])))
    cwd = listener.root_node
    logging.info('*** done: login ***')

    # Who am I.
    logging.info('*** start: whoami ***')
    logging.info('My email: {}'.format(api.getMyEmail()))
    executor.do(api.getAccountDetails, ())
    logging.info('*** done: whoami ***')

    # Make a directory.
    logging.info('*** start: mkdir ***')
    print '###', cwd.getName()
    check = api.getNodeByPath('sandbox', cwd)
    if check == None:
        executor.do(api.createFolder, ('sandbox', cwd))
    else:
        logging.info('Path already exists: {}'
                     .format(api.getNodePath(check)))
    logging.info('*** done: mkdir ***')

    # Now go and play in the sandbox.
    logging.info('*** start: cd ***')
    node = api.getNodeByPath('sandbox', cwd)
    if node == None:
        logging.warn('No such file or directory: sandbox')
    if node.getType() == MegaNode.TYPE_FOLDER:
        cwd = node
    else:
        logging.warn('Not a directory: sandbox')
    logging.info('*** done: cd ***')

    # Upload a file (create).
    logging.info('*** start: upload ***')
    executor.do(api.startUpload, ('README.md', cwd, None, 0 , None, False, False, None))
    logging.info('*** done: upload ***')

    # Download a file (read).
    logging.info('*** start: download ***')
    node = api.getNodeByPath('README.md', cwd)
    if node != None:
        executor.do(api.startDownload, (node, 'README_returned.md', None, None, False, None, MegaTransfer.COLLISION_CHECK_FINGERPRINT, MegaTransfer.COLLISION_RESOLUTION_NEW_WITH_N))
    else:
        logging.warn('Node not found: {}'.format('README.md'))
    logging.info('*** done: download ***')

    # Change a file (update).
    # Note: A new upload won't overwrite, but create a new node with same
    #       name!
    logging.info('*** start: update ***')
    old_node = api.getNodeByPath('README.md', cwd)
    executor.do(api.startUpload, ('README.md', cwd, None, 0 , None, False, False, None))
    if old_node != None:
        # Remove the old node with the same name.
        executor.do(api.remove, (old_node,))
    else:
        logging.info('No old file node needs removing')
    logging.info('*** done: update ***')

    # Delete a file.
    logging.info('*** start: delete ***')
    node = api.getNodeByPath('README.md', cwd)
    if node != None:
        executor.do(api.remove, (node,))
    else:
        logging.warn('Node not found: README.md')
    logging.info('*** done: delete ***')

    # Logout.
    logging.info('*** start: logout ***')
    executor.do(api.logout, ())
    listener.root_node = None
    logging.info('*** done: logout ***')


def main():
    """
    Sets up all that is needed to make (asynchronous) calls on the Mega API,
    then executs the worker to perform the desired operations.
    """
    # Get credentials.
    logging.info('Obtaining Mega login credentials.')
    credentials = {}
    if os.path.exists(CREDENTIALS_FILE):
        credentials = json.load(open(CREDENTIALS_FILE))
    else:
        credentials['user'] = raw_input('User: ')
        credentials['password'] = getpass.getpass()
    
    # Create the required Mega API objects.
    executor = AsyncExecutor()
    api = MegaApi(APP_KEY, None, None, 'Python CRUD example')
    listener = AppListener(executor.continue_event)
    api.addListener(listener)

    # Run the operations.
    start_time = time.time()
    worker(api, listener, executor, credentials)
    logging.info('Total time taken: {} s'.format(time.time() - start_time))


if __name__ == '__main__':
    # Set up logging.
    logging.basicConfig(level=logging.INFO,
                        #filename='runner.log',
                        format='%(levelname)s\t%(asctime)s (%(threadName)-10s) %(message)s')

    # Do the work.
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test runner that will limit the output of tests to a result set relevant for a
particular diff in Git. That is a diff between revisions or a diff between two
branches' tips.

This test runner will work with both Python 2.7 as well as 3.x.
"""

## Created: 23 May 2015 Guy Kloss <gk@mega.co.nz>
##
## (c) 2015 by Mega Limited, Auckland, New Zealand
##     http://mega.co.nz/
##     Simplified (2-clause) BSD License.
##
## You should have received a copy of the license along with this
## program.
##
## This file is part of the multi-party chat encryption suite.
##
## This code is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

__author__ = 'Guy Kloss <gk@mega.co.nz>'

import argparse
import os
import sys
import re
import logging
import tempfile
import subprocess
import collections
import config

PLATFORMS = {'posix': 'posix',
             'nt': 'win32'}
PROJECT_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                            os.path.pardir))
PATH_SPLITTER = re.compile(r'\\|/')


def get_git_line_sets(base, target):
    """
    Obtains the Git diff between the base and target to identify the lines that
    contain alterations in the target code. If branch names are given for the
    base or target, the tips of those branches are used.

    :param base: Base branch/commit for the diff.
    :param target: Target branch/commit for the diff.
    :return: A dictionary of changed line numbers. The key in the dictionary is
        the file path, the value is a set of line numbers.
    """
    # Get the Git output for the desired diff.
    logging.info('Extracting relevant lines from Git diff ...')
    command = 'git diff -U0 {} {}'.format(base, target)
    output = subprocess.check_output(command.split())
    diff = output.decode('latin1').split('\n')

    # Hunt down lines of changes for different files.
    file_line_mapping = collections.defaultdict(set)
    current_file = None
    for line in diff:
        if line.startswith('+++'):
            # Line giving target file.
            for_file = line.split()[1]
            current_file = tuple(re.split(PATH_SPLITTER, for_file[2:]))
        elif line.startswith('@@'):
            # Line giving alteration line range of diff fragment.
            target_lines = line.split()[2].split(',')
            start_line = int(target_lines[0])
            line_range = int(target_lines[1]) if len(target_lines) == 2 else 1
            # Update our lines if we have target lines.
            if line_range > 0:
                file_line_mapping[current_file].update(range(start_line,
                                                             start_line + line_range - 1))

    return file_line_mapping


def reduce_jshint(file_line_mapping, **extra):
    """
    Runs JSHint on the project with the default configured rules. The output
    is reduced to only contain entries from the Git change set.

    :param file_line_mapping: Mapping of files with changed lines (obtained
        `get_git_line_sets()`).
    :param extra: Optional keyword arguments:
        `norules`: If true, omit verbose output of violated rule identifier
                   (default: `False` to include rules).
    :return: A tuple containing the formatted string suitable for output and
        an integer containing the number of failed rules.
    """
    norules = extra['norules'] if 'norules' in extra else False
    # Get the JSHint output.
    logging.info('Obtaining JSHint output ...')
    os.chdir(PROJECT_PATH)
    rules = config.JSHINT_RULES if not norules else ''
    command = config.JSHINT_COMMAND.format(binary=config.JSHINT_BIN,
                                           rules=rules)
    output = None
    try:
        output = subprocess.check_output(command.split())
    except subprocess.CalledProcessError as ex:
        # JSHint found something, so it has returned an error code.
        # But we still want the output in the same fashion.
        output = ex.output
    output = output.decode('utf8').split('\n')

    # Go through output and collect only relevant lines to the result.
    result = ['\nJSHint output:\n==============\n']
    jshint_expression = re.compile(r'(.+): line (\d+), col \d+, .+')
    for line in output:
        parse_result = jshint_expression.findall(line)
        # Check if we've got a relevant line.
        if parse_result:
            file_name, line_no = parse_result[0][0], int(parse_result[0][1])
            file_name = tuple(re.split(PATH_SPLITTER, file_name))
            # Check if the line is part of our selection list.
            if line_no in file_line_mapping[file_name]:
                result.append(line)

    # Add the number of errors and return in a nicely formatted way.
    error_count = len(result) - 1
    result.append('\n{} errors'.format(error_count))
    return '\n'.join(result), error_count


def reduce_jscs(file_line_mapping, **extra):
    """
    Runs JSHCS on the project with the default configured rules. The output
    is reduced to only contain entries from the Git change set.

    :param file_line_mapping: Mapping of files with changed lines (obtained
        `get_git_line_sets()`).
    :param extra: Optional keyword arguments:
        `norules`: If true, omit verbose output of violated rule identifier
                   (default: `False` to include rules).
    :return: A tuple containing the formatted string suitable for output and
        an integer containing the number of failed rules.
    """
    norules = extra['norules'] if 'norules' in extra else False
    # Get the JSCS output.
    logging.info('Obtaining JSCS output ...')
    os.chdir(PROJECT_PATH)
    rules = config.JSHINT_RULES if not norules else ''
    command = config.JSCS_COMMAND.format(binary=config.JSCS_BIN,
                                         rules=rules)
    output = None
    try:
        output = subprocess.check_output(command.split())
    except subprocess.CalledProcessError as ex:
        # JSCS found something, so it has returned an error code.
        # But we still want the output in the same fashion.
        output = ex.output
    output = output.decode('utf8').split('\n\n')

    # Go through output and collect only relevant lines to the result.
    result = ['\nJSCS output:\n============']
    lines_expression = re.compile(r'^ +(\d+) |.*(?:\n|\r\n?)-', re.MULTILINE)
    file_expression = re.compile(r'^[^\b].* (?:\./)?(.+) :$', re.MULTILINE)
    for item in output:
        # Do the processing for every block here.
        line_no_candidates = lines_expression.findall(item, re.MULTILINE)
        # Check if we've got a relevant block.
        if line_no_candidates and '' in line_no_candidates:
            line_no = int(line_no_candidates[line_no_candidates.index('') - 1])
            file_name = file_expression.findall(item)[0]
            file_name = tuple(re.split(PATH_SPLITTER, file_name))
            # Check if the line is part of our selection list.
            if line_no in file_line_mapping[file_name]:
                result.append(item)

    # Add the number of errors and return in a nicely formatted way.
    error_count = len(result) - 1
    result.append('\n{} code style errors found.'.format(error_count))
    return '\n\n'.join(result), error_count


def reduce_cppcheck(file_line_mapping, **extra):
    """
    Runs CppCheck on the project with the default configured rules. The output
    is reduced to only contain entries from the Git change set.

    :param file_line_mapping: Mapping of files with changed lines (obtained
        `get_git_line_sets()`).
    :param extra: Optional keyword arguments:
        `platform`: A specific platform to test for, as used in
                    `include/mega/` (default: detect local system's platform).
    :return: A tuple containing the formatted string suitable for output and
        an integer containing the number of failed rules.
    """
    logging.info('Obtaining CppCheck output ...')
    platform = platform = PLATFORMS[os.name]
    if 'platform' in extra:
        # Override if given.
        platform = extra['platform']
        
    # Get the CppCheck output.
    os.chdir(PROJECT_PATH)
    command = config.CPPCHECK_COMMAND.format(command=config.CPPCHECK_BIN,
                                             platform=platform)
    output = None
    try:
        output = subprocess.check_output(command.split(),
                                         stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as ex:
        # CppCheck found something, so it has returned an error code.
        # But we still want the output in the same fashion.
        output = ex.output
    output = output.decode('utf8').split('\n')
    
    # Go through output and collect only relevant lines to the result.
    logging.debug('Reducing CppCheck output ...')
    result = ['\nCppCheck output:\n================\n']
    cppcheck_expression = re.compile(r'^(.+);(\d+);')
    for line in output:
        parse_result = cppcheck_expression.findall(line)
        # Check if we've got a relevant line.
        if parse_result:
            file_name, line_no = parse_result[0][0], int(parse_result[0][1])
            file_name = tuple(re.split(PATH_SPLITTER, file_name))
            # Check if the line is part of our selection list.
            if line_no in file_line_mapping[file_name]:
                formatted = '; '.join(line.split(';'))
                result.append(formatted)

    # Add the number of errors and return in a nicely formatted way.
    error_count = len(result) - 1
    result.append('\n{} errors'.format(error_count))
    return '\n'.join(result), error_count


def reduce_nsiqcppstyle(file_line_mapping, **extra):
    """
    Runs N'SIQ CppStyle on the project with the project configured rules.
    Thet output is reduced to only contain entries from the Git change set.

    :param file_line_mapping: Mapping of files with changed lines (obtained
        `get_git_line_sets()`).
    :param extra: Optional keyword arguments.
    :return: A tuple containing the formatted string suitable for output and
        an integer containing the number of failed rules.
    """
    logging.info("Obtaining N'SIQ CppStyle output ...")
    
    # Get the output.
    outfile = tempfile.mktemp()
    os.chdir(PROJECT_PATH)
    command = config.NSIQCPPSTYLE_COMMAND.format(binary=config.NSIQCPPSTYLE_BIN,
                                                 outfile=outfile)

    # Just capturing the console output to keep things quiet.
    consoleoutput = None
    try:
        consoleoutput = subprocess.check_output(command.split(),
                                                stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as ex:
        # CppCheck found something, so it has returned an error code.
        # But we still want the output in the same fashion.
        consoleoutput = ex.output

    # Get the output and delete the outfile.
    output = open(outfile, 'rt').read().split('\n')
    os.remove(outfile)
    
    # Go through output and collect only relevant lines to the result.
    logging.debug("Reducing N'SIQ CppStyle output ...")
    result = ["\nN'SIQ CppStyle output:\n======================\n"]
    # This stunt is required to fix Windows path problems with backslashes.
    base_path = os.path.join(PROJECT_PATH, '')
    if os.name == 'nt':
        base_path = base_path.replace('\\', '\\\\')
    nsiqcppstyle_expression = re.compile(r'^{}(.+?),(\d+),(.+),'
                                         .format(base_path))
    for line in output:
        parse_result = nsiqcppstyle_expression.findall(line)
        # Check if we've got a relevant line.
        if parse_result:
            file_name = tuple(re.split(PATH_SPLITTER, parse_result[0][0]))
            line_no = int(parse_result[0][1])
            rest = parse_result[0][2]
            # Check if the line is part of our selection list.
            if line_no in file_line_mapping[file_name]:
                formatted = ', '.join(['/'.join(file_name), str(line_no)]
                                      + rest.split(','))
                result.append(formatted)

    # Add the number of errors and return in a nicely formatted way.
    error_count = len(result) - 1
    result.append('\n{} errors'.format(error_count))
    return '\n'.join(result), error_count


def reduce_verapp(file_line_mapping, **extra):
    """
    Runs Vera++ on the project with the project configured rules. The output
    is reduced to only contain entries from the Git change set.

    :param file_line_mapping: Mapping of files with changed lines (obtained
        `get_git_line_sets()`).
    :param extra: Optional keyword arguments.
    :return: A tuple containing the formatted string suitable for output and
        an integer containing the number of failed rules.
    """
    logging.info('Obtaining Vera++ output ...')

    # Collect all C++ source files.
    cpp_source_expression = re.compile(r'\.(h|cpp)$')
    source_files = [os.path.join(dirpath, name)
                    for dirpath, _, items in os.walk(PROJECT_PATH)
                    for name in items
                    if cpp_source_expression.findall(name.lower())]

    # Get the Vera++ output.
    os.chdir(PROJECT_PATH)
    command = config.VERAPP_COMMAND.format(command=config.VERAPP_BIN,
                                           rules=' '.join(['-R {}'.format(item)
                                                           for item in config.VERAPP_RULES]))
    output = None
    try:
        process = subprocess.Popen(command.split(),
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.STDOUT,
                                   stdin=subprocess.PIPE)
        output, _ = process.communicate(bytes('\n'.join(source_files)
                                        .encode('ASCII')))
    except subprocess.CalledProcessError as ex:
        # Vera++ found something, so it has returned an error code.
        # But we still want the output in the same fashion.
        output = ex.output
    output = output.decode('utf8').split('\n')

    # Go through output and collect only relevant lines to the result.
    result = ['\nVera++ output:\n==============\n']
    # This stunt is required to fix Windows path problems with backslashes.
    base_path = os.path.join(PROJECT_PATH, '')
    if os.name == 'nt':
        base_path = base_path.replace('\\', '\\\\')
    verapp_expression = re.compile(r'^{}(.+?):(\d+): '.format(base_path))
    for line in output:
        parse_result = verapp_expression.findall(line)
        # Check if we've got a relevant line.
        if parse_result:
            file_name, line_no = parse_result[0][0], int(parse_result[0][1])
            file_name = tuple(re.split(PATH_SPLITTER, file_name))
            # Check if the line is part of our selection list.
            # TODO: find proper file name sub-set (remove base path)
            if line_no in file_line_mapping[file_name]:
                result.append(line)

    # Add the number of errors and return in a nicely formatted way.
    error_count = len(result) - 1
    result.append('\n{} errors'.format(error_count))
    return '\n'.join(result), error_count
    

def main(base, target, norules):
    """
    Run the JSHint and JSCS tests and present output ont eh console via print.
    """
    CHECKER_MAPPING = {'jshint': reduce_jshint,
                       'jscs': reduce_jscs,
                       'cppcheck': reduce_cppcheck,
                       'nsiqcppstyle': reduce_nsiqcppstyle,
                       'vera++': reduce_verapp}
    
    file_line_mapping = get_git_line_sets(base, target)
    results = []
    total_errors = 0
    for checker in config.checkers:
        worker = CHECKER_MAPPING[checker]
        extra_options = config.extra_options[checker]
        output, error_count = worker(file_line_mapping, **extra_options)
        results.append(output)
        total_errors += error_count
    logging.info('Output of reduced results ...')
    print('\n\n'.join(results))
    if total_errors > 0:
        sys.exit(1)


if __name__ == '__main__':
    # Set up logging.
    logging.basicConfig(level=logging.DEBUG,
                        format='%(levelname)s\t%(asctime)s %(message)s')

    # Setup the command line argument parser.
    DESCRIPTION = ('Filter output from static code analyser and style checker '
                   'to only contain content relevant to a diff '
                   '(e. g. between commits or tips of branches). '
                   'This tool will filter output from JSHint and JSCS.')
    EPILOG = 'Note: if no revision (commit ID) is given, the branch tip will be used.'
    parser = argparse.ArgumentParser(description=DESCRIPTION, epilog=EPILOG)
    parser.add_argument('--norules', default=False, action='store_true',
                        help="Don't show rule names with description (default: show rules names)")
    parser.add_argument('base',
                        help='base revision or name of base branch')
    parser.add_argument('target', nargs='?', default='',
                        help=('target revision or name of target branch'
                              ' (default: tip of current branch)'))

    args = parser.parse_args()

    main(args.base, args.target, args.norules)




# Extact a link from an email.
# Used by sdk/test/integration test SdkTest.SdkTestCreateAccount

import email
import imaplib
import os
import sys
import time


class EmailProcessor:
    mail = None
    debug = False

    def __init__(self, user, password, host='mail.mega.co.nz', port=993):
        self.mail = imaplib.IMAP4_SSL(host, port)
        self.mail.login(user, password)
        self.mail.select('Inbox')

    def get_validation_link_from_email(self, to, intent, delta=360):
        """Get validation link from email."""
        link = None
        messages = self.get_message_from_email(to, delta)
        if not messages:
            return None
        for message in messages:
            sub = list(message)[0]
            text = message[sub]
            if not text[1]:
                continue
            for line in text[1].splitlines():
                if self.debug: 
                    if line.startswith('https://'):
                        print("line:" + line)
                if line.startswith('https://') and ('#' + intent) in line:
                    link = line.strip()
                    break
        self.mail.close()
        self.mail.logout()
        return link

    def get_message_from_email(self, to, delta=360):
        """Get message from email."""

        # sort all emails sent to the right address, newest first
        result, data = self.mail.sort('REVERSE DATE', 'UTF-8', '(To "{}")'.format(to))
        if len(data[0]) == 0:
            return None

        # get all potential emails
        messages = []
        for n in data[0].split():
            _, d = self.mail.fetch(n, "(RFC822)")
            body = d[0][1]
            msg = email.message_from_string(body.decode('utf-8'))
            subject = msg['subject']
            if self.debug: print("subject: " + subject)
            text = None
            dt = 0
            for item in msg['DKIM-Signature'].split(';'):
                if 't=' in item:
                    dt = item.strip()[2:]
                    break
            else:
                assert dt, 'timestamp not found from email header'
            elapsed = time.time() - float(dt)
            if elapsed > delta:
                continue
            for m in msg.walk():
                content_type = m.get_content_type()
                if content_type == 'text/plain':
                    text = m.get_payload(decode=True).decode('raw-unicode-escape')
                    break
            if text:
                messages.append({subject: [n, text]})
        return messages

if len(sys.argv) == 1 or "--help" in sys.argv[1:] or (os.name == "nt" and "/?" in sys.argv[1:]):
    # no args, --help or /? on windows
    print("usage: " + sys.argv[0] + " email-user email-password to-email-address {confirm|cancel|recover|<other>} max-age-in-seconds")
    print("")
    print("e.g. python email_processor.py sdk-jenkins a-password sdk-jenkins+test-e-1@mega.co.nz recover 36000")
    print("$TEST_PASS can override email-password, but password placholder still requried on command line")
    exit(0);

user = os.getenv('TEST_USER') or sys.argv[1]
password = os.getenv('TEST_PASS') or sys.argv[2]
to = sys.argv[3]
intent = sys.argv[4]
if intent == "delete":
    # backwards compatible
    intent = "cancel"
    
delta = float(sys.argv[5])

ep = EmailProcessor(user, password)
link = ep.get_validation_link_from_email(to, intent, delta)
if link:
    print(link, end = '')


"""
 Application for stress testing syncing algorithm

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

import sys
import os
from threading import Thread
import random
import time
import string
import shutil
import logging
import collections

class Action:
    dir_type = "dir"
    file_type = "file"

    def __init__ (self, base_sync_dir, tmp_dir, otype):
        self.base_sync_dir = base_sync_dir
        self.otype = otype
        self.tmp_dir = tmp_dir
        self.seed = "MegaSDK"
        self.words = open("lorem.txt", "r").read().replace("\n", '').split()
        self.max_files = 100
        self.full_name = ""
        self.base_name = ""

    def is_dir (self):
        return self.otype == self.dir_type

    def is_file (self):
        return self.otype == self.file_type

    def get_random_str (self, size=10, chars = string.ascii_uppercase + string.digits):
        return ''.join (random.choice (chars) for x in range (size))

    def fdata (self):
        """
        Produce random data
        """
        a = collections.deque (self.words)
        b = collections.deque (self.seed)
        while True:
            yield ' '.join (list (a)[0:1024])
            a.rotate (int (b[0]))
            b.rotate (1)


    def create_file (self, fname, flen):
        """
        Create a file with the given length
        """
        g = self.fdata ()
        fout = open (fname, 'w')
        while os.path.getsize (fname) < flen:
            fout.write (g.next())
        fout.close ()

    def create (self):
        """
        Create a file / dir with a random name
        Must be the first action to be called
        """
        # generate a random file / dir name
        self.base_name = self.get_random_str ()
        # set a full name
        self.full_name = self.base_sync_dir + "/" + self.base_name

        if self.is_dir ():
            try:
                os.mkdir (self.full_name);
            except Exception, e:
                print "Failed to create dir !", e
                sys.exit (1)
        else:
            try:
                f = open (self.full_name, 'w')
            except Exception, e:
                print "Failed to create file !", e
                sys.exit (1)
            f.close ()
        logging.info ('[create] [%s] %s', self.otype, self.full_name)

    def rename (self):
        """
        Rename a file / dir
        """
        self.base_name = self.get_random_str ()
        new_name = self.base_sync_dir + "/" + self.base_name

        try:
            shutil.move (self.full_name, new_name)
        except Exception, e:
            print "Failed to move an object !", e
            sys.exit (1)
        logging.info ('[rename] [%s] %s => %s', self.otype, self.full_name, new_name)
        self.full_name = new_name

    def moveout (self):
        """
        Move out of sync dir
        """
        new_name = self.tmp_dir + "/" + self.base_name

        try:
            shutil.move (self.full_name, new_name)
        except Exception, e:
            print "Failed to move an object !", e
            sys.exit (1)

        logging.info ('[moveout] [%s] %s => %s', self.otype, self.full_name, new_name)
        self.full_name = new_name


    def movein (self):
        """
        Move into sync dir
        """
        new_name = self.base_sync_dir + "/" + self.base_name

        try:
            shutil.move (self.full_name, new_name)
        except Exception, e:
            print "Failed to move an object !", e
            sys.exit (1)

        logging.info ('[movein] [%s] %s => %s', self.otype, self.full_name, new_name)
        self.full_name = new_name

    def delete (self):
        """
        Delete file / dir
        """
        if self.is_dir ():
            try:
                shutil.rmtree (self.full_name)
            except Exception, e:
                print "Failed to delete directory !", e
                sys.exit (1)
        else:
            try:
                os.unlink (self.full_name)
            except Exception, e:
                print "Failed to delete file !", e
                sys.exit (1)
        logging.info ('[delete] [%s] %s', self.otype, self.full_name)
        self.full_name = ""
        self.base_name = ""

    def filldir (self):
        """
        fill dir with random files
        """
        if self.is_file ():
            return

        max_files = random.randint (1, self.max_files)
        for i in range (0, max_files):
            flen = random.randint (1, 1024 * 4)
            fname = self.get_random_str ()
            self.create_file (self.full_name + "/" + fname, flen)

        logging.info ('[filldir] [%s] created %d files', self.otype, max_files)


class Worker (Thread):
    """
    Worker class (run in a thread)
    """
    def __init__ (self, base_sync_dir, tmp_dir):
        Thread.__init__(self)
        self.base_sync_dir = base_sync_dir
        self.tmp_dir = tmp_dir

    def cmds (self):
        f = open ("cmds.txt", "r")
        l_cmds = []
        for line in f:
            if (line[0] != '#') and (len (line) > 1):
                l_cmds.append (line.strip())
        f.close()
        i = 0
        while i < len (l_cmds):
            yield l_cmds[i]
            i = i + 1

    def run (self):
        otype = [Action.dir_type, Action.file_type]
        a = Action (self.base_sync_dir, self.tmp_dir, random.choice (otype))
        cmd = self.cmds ()
        for c in cmd:
            if c == "create":
                a.create ()
            elif c == "moveout":
                a.moveout ()
            elif c == "movein":
                a.movein ()
            elif c == "rename":
                a.rename ()
            elif c == "filldir":
                a.filldir ()
            elif c == "delete":
                a.delete ()
            elif c == "sleep":
                logging.info ('[sleep]')
                time.sleep (2)
            else:
                logging.error ('Unknown command: %s', c)


def main (num_workers, base_sync_dir, tmp_dir):
    random.seed (time.time())

    w_list = []
    for i in range (0, num_workers):
        w = Worker (base_sync_dir, tmp_dir)
        w_list.append (w)

    for w in w_list:
        w.run ()

if __name__ == "__main__":
    if len (sys.argv) < 4:
        print "Please run as: " + sys.argv[0] + " [number of workers] [sync directory] [out of sync directory]"
        sys.exit (1)

    # logging stuff, output to stdout
    logging.StreamHandler (sys.stdout)
    logging.basicConfig (level=logging.INFO)

    main (int (sys.argv[1]), sys.argv[2], sys.argv[3])



# -*- coding: utf-8 -*-
"""
Configuration file for the `diffcheck.py` tool.

This file should be checked into the repository configuring the
project-specific behaviour. Local (user-specific) modifications can be
made by creating a `local_config.py` file in the same directory as this,
which can contain options overlaying those specified for the proejct
here.
"""

import os

# Checkers to run. Available options for checkers are:
# "jshint", "jscs", "cppcheck", "nsiqcppstyle", "vera++"
checkers = ['vera++', 'nsiqcppstyle', 'cppcheck']

# Extra options for designated checkers.
# This configuration needs an entry for every encountered checker if the
# `checkers` option above.
extra_options = {
    'jshint': {'norules': False},
    'jscs': {'norules': False},
    'cppcheck': {},
    'nsiqcppstyle': {},
    'vera++': {}
}

# Paths for the executables to use.
JSHINT_BIN = 'node_modules/.bin/jshint'
JSCS_BIN = 'node_modules/.bin/jscs'
CPPCHECK_BIN = 'cppcheck'
VERAPP_BIN = 'vera++'

NSIQCPPSTYLE_BIN = '/usr/local/nsiqcppstyle/nsiqcppstyle.py'
JSHINT_RULES = '--verbose'
JSCS_RULES = '--verbose'
# Vera++ rules like this should be superseded by a "profile", but it
# doesn't work well, yet, on Vera++ v1.2 :-(
# For rules, look here:
# https://bitbucket.org/verateam/vera/wiki/Rules
VERAPP_RULES = ['F001', 'F002',
                'L001', 'L002', 'L003', 'L004', 'L005',
                'T001', 'T002', 'T003', 'T004', 'T005', 'T006', 'T007',
                'T008', 'T009', 'T010', 'T011', 'T013', 'T017', 'T018',
                'T019']


# Command line configuration.
JSHINT_COMMAND = '{binary} {rules} .'
JSCS_COMMAND = '{binary} {rules} .'

CPPCHECK_COMMAND = ("{command}"
                    " --template={{file}};{{line}};{{severity}};{{id}};{{message}}"
                    " --enable=warning,portability,information,missingInclude"
                    " --std=c++03 --force"
                    " --quiet"
                    " -I include"
                    " -I include/mega/{platform}"
                    " src/ examples/")
NSIQCPPSTYLE_COMMAND = ('python {binary} --output=csv --ci -o {outfile}'
                        ' -f contrib/nsiq_filefilter.txt .')
VERAPP_COMMAND = ('vera++ --show-rule --summary'
                  ' {rules}'
                  ' --parameter max-line-length=120 -i -')


# Some attempts to "auto fix" stuff for Win.
if os.name == 'nt':
    JSHINT_BIN = '{}.cmd'.format(JSHINT_BIN).replace('/', '\\')
    JSCS_BIN = '{}.cmd'.format(JSCS_BIN).replace('/', '\\')
    CPPCHECK_BIN += '.exe'
    VERAPP_BIN += '.exe'
    JSHINT_COMMAND = 'cmd /c {}'.format(JSHINT_COMMAND)
    JSCS_COMMAND = 'cmd /c {}'.format(JSCS_COMMAND)

# Overlay project-config with a potentially available local configuration.
try:
    from local_config import *
except ImportError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Simplistic test module for the Python bindings."""

## Created: 14 Jan 2015 Paul Ionkin <pi@mega.co.nz>
##
## (c) 2015 by Mega Limited, Auckland, New Zealand
##     http://mega.co.nz/
##     Simplified (2-clause) BSD License.
##
## You should have received a copy of the license along with this
## program.
##
## This file is part of the multi-party chat encryption suite.
##
## This code is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

__author__ = 'Paul Ionkin <pi@mega.co.nz>'

import sys
import os
import pprint

_here = os.path.abspath(os.path.dirname(__file__))
_wrapper_dir = os.path.join(_here, '..', '..', 'bindings', 'python')
_libs_dir = os.path.join(_wrapper_dir, '.libs')
_shared_lib = os.path.join(_libs_dir, '_mega.so')
if os.path.isdir(_wrapper_dir) and os.path.isfile(_shared_lib):
    sys.path.insert(0, _wrapper_dir)  # mega.py
    sys.path.insert(0, _libs_dir)     # _mega.so

import mega

# TODO: extend test example
def main():
    api = mega.MegaApi("test")
    pprint.pprint(dir(api))

if __name__ == '__main__':
    main()



"""
 Application for testing syncing algorithm

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

import sys
import os
import time
import subprocess
from sync_test_base import get_random_str
from sync_test import SyncTest
from sync_test_app import SyncTestApp
import unittest
import xmlrunner
import logging
import argparse
import platform
import math


class SyncTestMegaSyncApp(SyncTestApp):
    """
    operates with megasync application
    """
    def __init__(self, work_dir, remote_folder, delete_tmp_files=True, use_large_files=True):
        """
        work_dir: a temporary folder to place generated files
        remote_folder: a remote folder to sync
        """

        self.megasync_ch_in = None
        self.megasync_ch_out = None

        self.local_mount_in = os.path.join(work_dir, "sync_in")
        self.local_mount_out = os.path.join(work_dir, "sync_out")

        self.work_dir = os.path.join(work_dir, "tmp")

        self.work_dir_in = os.path.join(work_dir, "tmp/in")
        self.work_dir_out = os.path.join(work_dir, "tmp/out")

        self.remote_folder = remote_folder

        # init base class
        super(SyncTestMegaSyncApp, self).__init__(self.local_mount_in, self.local_mount_out, self.work_dir, delete_tmp_files, use_large_files)

        try:
            os.makedirs(self.local_mount_in)
        except OSError:
            pass

        try:
            os.makedirs(self.local_mount_out)
        except OSError:
            pass

        try:
            os.makedirs(self.work_dir)
        except OSError:
            pass

        try:
            os.makedirs(self.work_dir_in)
        except OSError:
            pass

        try:
            os.makedirs(self.work_dir_out)
        except OSError:
            pass


        if not os.access(self.local_mount_in, os.W_OK | os.X_OK):
            raise Exception("Not enough permissions to create / write to directory")

        if not os.access(self.local_mount_out, os.W_OK | os.X_OK):
            raise Exception("Not enough permissions to create / write to directory")

        if not os.access(self.work_dir, os.W_OK | os.X_OK):
            raise Exception("Not enough permissions to create / write to directory")

    def start_megasync(self, local_folder, app_wkd, type_str):
        """
        fork and launch "megasimplesync" application
        local_folder: local folder to sync
        """
        # launch megasimplesync
        base_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), '../..')

        # the app is either in examples/ or in the project's root
        if platform.system() == "Windows":
            app_name = "megasimplesync.exe"
        else:
            app_name = "megasimplesync"
        bin_path = os.path.join(base_path, "examples")
        tmp = os.path.join(bin_path, app_name)
        if not os.path.isfile(tmp) or not os.access(tmp, os.X_OK):
            bin_path = os.path.join(base_path, "")
            tmp = os.path.join(bin_path, app_name)
            if not os.path.isfile(tmp) or not os.access(tmp, os.X_OK):
                raise Exception("megasimplesync application is not found!")

        pargs = [os.path.join(bin_path, app_name), local_folder, self.remote_folder]
        output_fname = os.path.join(self.work_dir, "megasimplesync" + "_" + type_str + "_" + get_random_str() + ".log")
        output_log = open(output_fname, "w")

        logging.info("Launching on: \"%s\", cmd: \"%s\", log: \"%s\"" % (app_wkd," ".join(pargs), output_fname))

        try:
            ch = subprocess.Popen(pargs, cwd=app_wkd, universal_newlines=True, stdout=output_log, stderr=subprocess.STDOUT, bufsize=1, shell=False, env=os.environ)
        except OSError, e:
            logging.error("Failed to launch megasimplesync process: %s" % e)
            return None
        return ch

    def sync(self):
        """
        TODO: wait for full synchronization
        """
        if not hasattr(self,'attempt'):
            self.attempt=self.nr_retries
        if (self.attempt==self.nr_retries):
            time.sleep(5)
        else:
            time.sleep(math.pow(1.0181780986123312,self.attempt+1)*0.5) #~200*5 secs

    def start(self):
        """
        prepare and run tests
        """

        if os.environ.get('MEGA_EMAIL') is None or os.environ.get('MEGA_PWD') is None:
            logging.error("Environment variables MEGA_EMAIL and MEGA_PWD are not set !")
            return False

        # start "in" instance
        self.megasync_ch_in = self.start_megasync(self.local_mount_in, self.work_dir_in, "in")
        # pause
        time.sleep(5)
        # start "out" instance
        self.megasync_ch_out = self.start_megasync(self.local_mount_out, self.work_dir_out, "out")
        # check both instances
        if self.megasync_ch_in is None or self.megasync_ch_out is None:
            return False

        return True

    def finish(self):
        """
        kill megasync instances, remove temp folders
        """

        self.sync()

        # kill instances
        for _ in range(0, 5):
            if self.megasync_ch_in:
                self.megasync_ch_in.terminate()
                self.megasync_ch_in.poll()
                if (self.megasync_ch_in.poll() is not None):
                    if self.megasync_ch_in.returncode is not None: logging.debug("IN process terminated due to signal "+ str(self.megasync_ch_in.returncode))
                    break
                logging.debug("waiting for IN process to end")
                time.sleep(5)
            else:
                break

        for _ in range(0, 5):
            if self.megasync_ch_out:
                self.megasync_ch_out.terminate()
                if (self.megasync_ch_out.poll() is not None):
                    if self.megasync_ch_out.returncode is not None: logging.debug("OUT process terminated due to signal "+ str(self.megasync_ch_out.returncode))
                    break
                logging.debug("waiting for OUT process to end")
                time.sleep(5)
            else:
                break

    def is_alive(self):
        """
        return True if application instance is running
        """
        if not self.megasync_ch_in or not self.megasync_ch_out:
            return False
        return self.megasync_ch_in.poll() is None and self.megasync_ch_out.poll() is None

    def pause(self):
        """
        pause application
        """

    def unpause(self):
        """
        unpause application
        """

if __name__ == "__main__":
    parser = argparse.ArgumentParser(epilog="Please set MEGA_EMAIL and MEGA_PWD environment variables.")
    parser.add_argument("--test1", help="test_create_delete_files", action="store_true")
    parser.add_argument("--test2", help="test_create_rename_delete_files", action="store_true")
    parser.add_argument("--test3", help="test_create_delete_dirs", action="store_true")
    parser.add_argument("--test4", help="test_create_rename_delete_dirs", action="store_true")
    parser.add_argument("--test5", help="test_sync_files_write", action="store_true")
    parser.add_argument("--test6", help="test_local_operations", action="store_true")
    parser.add_argument("--test7", help="test_update_mtime", action="store_true")
    parser.add_argument("--test8", help="test_create_rename_delete_unicode_files_dirs", action="store_true")
    parser.add_argument("--test9", help="test_create_move_delete_files", action="store_true")
    parser.add_argument("--test10", help="test_mimic_update_with_backup_files", action="store_true")
    parser.add_argument("-a", "--all", help="run all tests", action="store_true")
    parser.add_argument("-b", "--basic", help="run basic, stable tests", action="store_true")
    parser.add_argument("-d", "--debug", help="use debug output", action="store_true")
    parser.add_argument("-l", "--large", help="use large files for testing", action="store_true")
    parser.add_argument("-n", "--nodelete", help="Do not delete work files", action="store_false")
    parser.add_argument("-f", "--files", type=int,help="Number of files")
    parser.add_argument("-c", "--folders", type=int,help="Number of folders")
    parser.add_argument("-g", "--changes", type=int,help="Number of changes")
    parser.add_argument("-r", "--retries", type=int,help="Number of retries")
    parser.add_argument("work_dir", help="local work directory")
    parser.add_argument("sync_dir", help="remote directory for synchronization")
    parser.add_argument("--only-empty-files", help="use only empty files", action="store_true")    
    args = parser.parse_args()

    if args.debug:
        lvl = logging.DEBUG
        # megasync will use verbose output
        os.environ["MEGA_DEBUG"] = "2"
    else:
        lvl = logging.INFO

    if args.all:
        args.test1 = args.test2 = args.test3 = args.test4 = args.test5 = args.test6 = args.test7 = args.test8 = args.test9 = args.test10 = True
    if args.basic:
        args.test1 = args.test2 = args.test3 = args.test4 = True

    # logging stuff, output to stdout
    logging.StreamHandler(sys.stdout)
    logging.basicConfig(format='[%(asctime)s] %(message)s', datefmt='%Y-%m-%d %H:%M:%S', level=lvl)


    with SyncTestMegaSyncApp(args.work_dir, args.sync_dir, args.nodelete, args.large) as app:
        suite = unittest.TestSuite()
        
        app.only_empty_files=args.only_empty_files
        app.only_empty_folders=False
        if args.retries is not None: 
            app.nr_retries=args.retries

        synctests=[]
        if args.test1:
            synctests.append(SyncTest("test_create_delete_files", app))

        if args.test2:
            synctests.append(SyncTest("test_create_rename_delete_files", app))

        if args.test3:
            synctests.append(SyncTest("test_create_delete_dirs", app))

        if args.test4:
            synctests.append(SyncTest("test_create_rename_delete_dirs", app))

        if args.test5:
            synctests.append(SyncTest("test_sync_files_write", app))

        if args.test6:
            synctests.append(SyncTest("test_local_operations", app))

        if args.test7:
            synctests.append(SyncTest("test_update_mtime", app))

        if args.test8:
            synctests.append(SyncTest("test_create_rename_delete_unicode_files_dirs", app))

        if args.test9:
            synctests.append(SyncTest("test_create_move_delete_files", app))
          
        if args.test10:
            synctests.append(SyncTest("test_mimic_update_with_backup_files", app))


        for st in synctests:
            if args.files is not None: 
                st.nr_files=args.files
                st.local_obj_nr=args.files
            if args.folders is not None: st.nr_dirs=args.folders
            if args.changes is not None: 
                st.nr_changes=args.changes
                st.nr_time_changes=args.changes
            if args.retries is not None: 
                st.nr_retries=args.retries

            suite.addTest(st)
            
            
        testRunner = xmlrunner.XMLTestRunner(output='test-reports')
        testRunner.run(suite)



"""
 Application for generating random Unicode filenames

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

# python unicode_filenames.py -o out -p -c 100 -m 0xFF -l 1

import random
import os
import platform
import string
import sys
import argparse

def touch(path):
    """
    create an empty file
    update utime
    """
    with open(path, 'a'):
        os.utime(path, None)

def gen_name(l, exclude, max_val):
    """
    Generate random string of size l
    """
    name = u""
    while len(name) < l:
        c = unichr(random.randint(0, max_val))
        if c not in exclude:
            name = name + c
    return name

class hexact(argparse.Action):
    'An argparse.Action that handles hex string input'
    def __call__(self, parser, namespace, values, option_string=None):
        base = 10
        if '0x' in values:
            base = 16
        setattr(namespace, self.dest, int(values, base))
        return

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Create files with random Unicode filenames.")
    parser.add_argument("-c", "--file-count", help="How many files to create, default is 10000 (not checking for unique names)", default=1000, type=int)
    parser.add_argument("-l", "--filename-length", help="Generated filenames length, default is 20", default=20, type=int)
    parser.add_argument("-m", "--max-unicode-char", help="Maximal Unicode character, default is 0xFFFF", action=hexact, default=0xFFFF)
    parser.add_argument("-o", "--output-dir", help="Output dir (required)")
    parser.add_argument("-p", "--toprint", help="print generated filename to stdout", action="store_true")
    args = parser.parse_args()

    if args.output_dir is None:
        parser.print_help()
        sys.exit(1)

    try:
        os.makedirs(args.output_dir)
    except OSError, e:
        pass

    if platform.system() == "Windows":
        # Unicode characters 1 through 31, as well as quote ("), less than (<), greater than (>), pipe (|), backspace (\b), null (\0) and tab (\t).
        exclude = string.punctuation + u"\t" +  u''.join([unichr(x) for x in range(0, 32)])
    else:
        # find . -exec rm {} \;
        exclude = u"/" + u"." + u''.join([unichr(x) for x in range(0, 1)])

    for _ in range(0, args.file_count):
        s = gen_name(args.filename_length, exclude, args.max_unicode_char)
        fname = os.path.join(args.output_dir, s)
        if args.toprint:
            print fname.encode('utf-8')
        touch(fname)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Simple command line interface using the Python bindings."""

## Created: 15 Jan 2015 Javier Serrano <js@mega.co.nz>
##
## (c) 2015 by Mega Limited, Auckland, New Zealand
##     http://mega.co.nz/
##     Simplified (2-clause) BSD License.
##
## You should have received a copy of the license along with this
## program.
##
## This file is part of the multi-party chat encryption suite.
##
## This code is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

__author__ = 'Javier Serrano <js@mega.co.nz>'

import sys
import os
import cmd
import logging
import time

from mega import (MegaApi, MegaListener, MegaError, MegaRequest,
                  MegaUser, MegaNode)


class AppListener(MegaListener):
    def __init__(self, shell):
        self._shell = shell
        super(AppListener, self).__init__()


    def onRequestStart(self, api, request):
        logging.info('Request start ({})'.format(request))


    def onRequestFinish(self, api, request, error):
        logging.info('Request finished ({}); Result: {}'
                     .format(request, error))
        if error.getErrorCode() != MegaError.API_OK:
            return

        request_type = request.getType()
        if request_type == MegaRequest.TYPE_LOGIN:
            api.fetchNodes()
        elif request_type == MegaRequest.TYPE_EXPORT:
            logging.info('Exported link: {}'.format(request.getLink()))
        elif request_type == MegaRequest.TYPE_ACCOUNT_DETAILS:
            account_details = request.getMegaAccountDetails()
            logging.info('Account details received')
            logging.info('Account e-mail: {}'.format(api.getMyEmail()))
            logging.info('Storage: {} of {} ({} %)'
                         .format(account_details.getStorageUsed(),
                                 account_details.getStorageMax(),
                                 100 * account_details.getStorageUsed()
                                 / account_details.getStorageMax()))
            logging.info('Pro level: {}'.format(account_details.getProLevel()))


    def onRequestTemporaryError(self, api, request, error):
        logging.info('Request temporary error ({}); Error: {}'
                .format(request, error))


    def onTransferFinish(self, api, transfer, error):
        logging.info('Transfer finished ({}); Result: {}'
                .format(transfer, transfer.getFileName(), error))


    def onTransferUpdate(self, api, transfer):
        logging.info('Transfer update ({} {});'
                     ' Progress: {} KB of {} KB, {} KB/s'
                     .format(transfer,
                             transfer.getFileName(),
                             transfer.getTransferredBytes() / 1024,
                             transfer.getTotalBytes() / 1024,
                             transfer.getSpeed() / 1024))


    def onTransferTemporaryError(self, api, transfer, error):
        logging.info('Transfer temporary error ({} {}); Error: {}'
                     .format(transfer, transfer.getFileName(), error))


    def onUsersUpdate(self, api, users):
        if users != None:
            logging.info('Users updated ({})'.format(users.size()))

    def onNodesUpdate(self, api, nodes):
        if nodes != None:
            logging.info('Nodes updated ({})'.format(nodes.size()))
        else:
            self._shell.cwd = api.getRootNode()


class MegaShell(cmd.Cmd, MegaListener):
    intro = 'Mega sample app. Type help or ? to list commands.\n'
    PROMPT = '(MEGA)'

    def __init__(self, api):
        self._api = api
        self.cwd = None
        super(MegaShell, self).__init__()


    def emptyline(self):
        return


    def do_login(self, arg):
        """Usage: login email password"""
        args = arg.split()
        if len(args) != 2 or '@' not in args[0]:
            print(self.do_login.__doc__)
            return

        self._api.login(args[0], args[1])


    def do_logout(self, arg):
        """Usage: logout"""

        args = arg.split()
        if len(args) != 0:
            print(self.do_logout.__doc__)
            return

        if self.cwd == None:
            print('INFO: Not logged in')
            return

        self._api.logout()
        self.cwd = None


    def do_mount(self, arg):
        """Usage: mount"""
        args = arg.split()
        if len(args) != 0:
            print(self.do_mount.__doc__)
            return
        if not self._api.isLoggedIn():
            print('INFO: Not logged in')
            return

        print('INFO: INSHARES:')
        users = self._api.getContacts()
        for i in range(users.size()):
            user = users.get(i)
            if user.getVisibility() == MegaUser.VISIBILITY_VISIBLE:
                shares = self._api.getInShares(user)
                for j in range(shares.size()):
                    share = shares.get(j)
                    print('INFO: INSHARE on {} {} Access level: {}'
                          .format(users.get(i).getEmail(),
                                  are.getName(),
                                  self._api.getAccess(share)))


    def do_ls(self, arg):
        """Usage: ls [path]"""
        args = arg.split()
        if len(args) > 1:
            print(self.do_ls.__doc__)
            return

        if self.cwd == None:
            print('INFO: Not logged in')
            return

        path = None
        if len(args) == 0:
            path = self.cwd
        else:
            path = self._api.getNodeByPath(args[0], self.cwd)

        print('    .')
        if self._api.getParentNode(path) != None:
            print('    ..')

        nodes = self._api.getChildren(path)
        for i in range(nodes.size()):
            node = nodes.get(i)
            output = '    {}'.format(node.getName())
            if node.getType() == MegaNode.TYPE_FILE:
                output += '   ({} bytes)'.format(node.getSize())
            else:
                output += '   (folder)'
            print(output)


    def do_cd(self, arg):
        """Usage: cd [path]"""
        args = arg.split()
        if len(args) > 1:
            print(self.do_cd.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        if len(args) == 0:
            self.cwd = self._api.getRootNode()
            return

        node = self._api.getNodeByPath(args[0], self.cwd)
        if node == None:
            print('{}: No such file or directory'.format(args[0]))
            return
        if node.getType() == MegaNode.TYPE_FILE:
            print('{}: Not a directory'.format(args[0]))
            return
        self.cwd = node


    def do_get(self, arg):
        """Usage: get remotefile"""
        args = arg.split()
        if len(args) != 1:
            print(self.do_get.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        node = self._api.getNodeByPath(args[0], self.cwd)
        if node == None:
            print('Node not found: {}'.format(args[0]))
            return

        self._api.startDownload(node, './', None, None, False, None, MegaTransfer.COLLISION_CHECK_FINGERPRINT, MegaTransfer.COLLISION_RESOLUTION_NEW_WITH_N)


    def do_put(self, arg):
        """Usage: put localfile"""
        args = arg.split()
        if len(args) != 1:
            print(self.do_put.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        self._api.startUpload(args[0], self.cwd, None, 0, None, False, False, None)


    def do_mkdir(self, arg):
        """Usage: mkdir path"""
        args = arg.split()
        if len(args) != 1:
            print(self.do_mkdir.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        base = self.cwd
        name = args[0]
        if '/' in name or '\\' in name:
            index1 = name.rfind('/')
            index2 = name.rfind('\\')
            index = None
            if index1 > index2:
                index = index1
            else:
                index = index2
            path = name[:index + 1]
            base = self._api.getNodeByPath(path, self.cwd)
            name = name[index + 1:]

            if not name:
                print('{}: Path already exists'.format(path))
                return
            if base == None:
                print('{}: Target path not found'.format(path))
                return

        check = self._api.getNodeByPath(name, base)
        if check != None:
            print('{}: Path already exists'
                  .format(self._api.getNodePath(check)))
            return

        self._api.createFolder(name, base)


    def do_rm(self, arg):
        """Usage: rm path"""
        args = arg.split()
        if len(args) != 1:
            print(self.do_rm.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        node = self._api.getNodeByPath(args[0], self.cwd)
        if node == None:
            print('Node not found: {}'.format(args[0]))
            return

        self._api.remove(node)


    def do_mv(self, arg):
        """Usage: mv srcpath dstpath"""
        args = arg.split()
        if len(args) != 2:
            print(self.do_mv.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        src_node = self._api.getNodeByPath(args[0], self.cwd)
        if src_node == None:
            print('{}: No such file or directory'.format(args[0]))
            return

        name = args[1]
        dst_node = self._api.getNodeByPath(name, self.cwd)
        if (dst_node != None) and (dst_node.getType() == MegaNode.TYPE_FILE):
            print('{}: Not a directory'.format(name))
            return

        if dst_node != None:
            self._api.moveNode(src_node, dst_node)
            return

        if '/' in name or '\\' in name:
            index1 = name.rfind('/')
            index2 = name.rfind('\\')
            index = None
            if index1 > index2:
                index = index1
            else:
                index = index2
            path = name[:index + 1]
            base = self._api.getNodeByPath(path, self.cwd)
            name = name[index + 1:]

            if base == None:
                print('{}: No such directory'.format(path))
                return

            if base.getType() == MegaNode.TYPE_FILE:
                print('{}: Not a directory'.format(path))
                return

            self._api.moveNode(src_node, base)
            if len(name) != 0:
                self._api.renameNode(src_node, name)
            return

        if dst_node == None:
            self._api.renameNode(src_node, name)
            return


    def do_pwd(self, arg):
        """Usage: pwd"""
        args = arg.split()
        if len(args) != 0:
            print(self.do_pwd.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        print('{} INFO: Current working directory: {}'
              .format(self.PROMPT, self._api.getNodePath(self.cwd)))


    def do_export(self, arg):
        """Usage: export path"""
        args = arg.split()
        if len(args) != 1:
            print(self.do_export.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        node = self._api.getNodeByPath(args[0], self.cwd)
        self._api.exportNode(node)


    def do_import(self, arg):
        """Usage: import exportedfilelink#key"""
        args = arg.split()
        if len(args) != 1:
            print(self.do_import.__doc__)
            return
        if self.cwd == None:
            print('INFO: Not logged in')
            return

        self._api.importFileLink(args[0], self.cwd)


    def do_whoami(self, arg):
        """Usage: whoami"""
        args = arg.split()
        if len(args) != 0:
            print(self.do_whoami.__doc__)
            return
        if not self._api.isLoggedIn():
            print('INFO: Not logged in')
            return
        print(self._api.getMyEmail())
        self._api.getAccountDetails()


    def do_passwd(self, arg):
        """Usage: passwd <old password> <new password> <new password>"""
        args = arg.split()
        if len(args) != 3:
            print(self.do_passwd.__doc__)
            return
        if not self._api.isLoggedIn():
            print('INFO: Not logged in')
            return

        if args[1] != args[2]:
            print('Mismatch, please try again')
            return

        self._api.changePassword(args[0], args[1])


    def do_quit(self, arg):
        """Usage: quit"""
        del self._api
        print('Bye!')
        return True


    def do_exit(self, arg):
        """Usage: exit"""
        del self._api
        print('Bye!')
        return True


if __name__ == '__main__':
    # Set up logging.
    logging.basicConfig(level=logging.INFO,
                        #filename='runner.log',
                        format='%(levelname)s\t%(asctime)s %(message)s')
    # Do the work.
    api = MegaApi('ox8xnQZL', None, None, 'Python megacli')
    shell = MegaShell(api)
    listener = AppListener(shell)
    api.addListener(listener)
    api = None
    shell.cmdloop()
    



"""
 Base class for testing syncing algorithm

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

import os
import random
import string
import shutil
import hashlib
import unittest
import logging
import platform
import unicodedata
import time

def get_unicode_str(size=10, max_char=0xFFFF, onlyNormalized=False, includeUnexisting=False):
    '''
    generates valid (for current OS) Unicode file name
    Notice: if includeUnexisting==True, it is possible that files don't get synchronized
    '''
    if platform.system() == "Windows":
        # Unicode characters 1 through 31, as well as quote ("), less than (<), greater than (>), pipe (|), backspace (\b), null (\0) and tab (\t).
        exclude = string.punctuation + u"\t" +  u''.join([unichr(x) for x in range(0, 32)])
    else:
        # I guess it mainly depends on fs type
        #exclude = u"/" + u"." + u''.join([unichr(x) for x in range(0, 1)])
        exclude = u"/" + u"." + u''.join([unichr(x) for x in range(0, 32)])


    name = u""
    while len(name) < size:
        c = unichr(random.randint(0, max_char))
        if c not in exclude:
            try:
                if not includeUnexisting:
                    unicodedata.name(c) #this will cause invalid unicode character to throw exception
                if onlyNormalized:
                    name = name + unicodedata.normalize('NFC',c) #only normalized chars
                else:
                    name = name + c
 #           except UnicodeDecodeError:
 #               print "UnicodeDecodeError con",c,repr(c),c.encode('utf-8')
 #               c.decode('utf-8')
 #               try:
 #                   unicodedata.name(c)
 #               except:
 #                   pass
 #               pass
            except ValueError:
 #               try:
 #                   unicodedata.name(c)
#                    print "that one was valid!",c,repr(c)
 #                   pass
 #               except:
 #                   pass
                pass
    return name

def get_exotic_str(size=10):
    """
    generate string containing random combinations of % and ASCII symbols
    """
    name = u""
    while len(name) < size:
        num = random.randint(1, 3)
        name = name + u"%" * num + get_random_str(2)
    return name

def get_random_str(size=10, chars=string.ascii_lowercase + string.ascii_uppercase + string.digits):
    """
    return a random string
    size: size of an output string
    chars: characters to use
    """
    return ''.join(random.choice(chars) for x in range(size))

def generate_ascii_name(first_symbol, i):
    """
    generate random ASCII string
    """
    strlen = random.randint(0, 20)
    return first_symbol + get_random_str(size=strlen) + str(i)

def generate_unicode_name_old(first_symbol, i):
    """
    generate random UTF string
    """
    strlen = random.randint(10, 30)
    c = random.choice(['short-utf', 'utf', 'exotic'])
    if c == 'short-utf':
        s = get_unicode_str(strlen, 0xFF)
    elif c == 'utf':
        s = get_unicode_str(strlen)
    else:
        s = get_exotic_str(strlen)
    #logging.debug("Creating Unicode file:  %s" % (s.encode("unicode-escape")))
    return s

cogen=0
def generate_unicode_name(first_symbol, i):
    """
    generate random UTF string
    """
    strlen = random.randint(10, 30)
    c = random.choice(['short-utf', 'utf', 'exotic'])
    if c == 'short-utf':
        s = get_unicode_str(strlen, 0xFF)
    elif c == 'utf':
        s = get_unicode_str(strlen)
    else:
        s = get_exotic_str(strlen)
    #logging.debug("Creating Unicode file:  %s" % (s.encode("unicode-escape")))
    global cogen
    cogen=cogen+1
    return str(cogen)+"_"+s

def normalizeandescape(name):
    name=escapefsincompatible(name)
    name=unicodedata.normalize('NFC',unicode(name))
    #name=unicodedata.normalize('NFC',name)
    #name=escapefsincompatible(name)
    return name

def escapefsincompatible(name):
    """
    Escape file system incompatible characters
    """
    import urllib
    for i in "\\/:?\"<>|*":
        name=name.replace(i,urllib.quote(i).lower())
    return name

class SyncTestBase(unittest.TestCase):
    """
    Base class with MEGA SDK test helper methods
    """
    def __init__(self, methodName, app):
        """
        local_mount_in: local upsync folder
        local_mount_out: local downsync folder
        work_folder: a temporary out of sync folder
        """
        super(SyncTestBase, self).__init__(methodName)

        self.app = app

        self.nr_retries = 200
        self.nr_files = 10
        self.nr_dirs = 10
        self.nr_time_changes = 10
        self.nr_changes = 10
        self.local_obj_nr = 5
        self.force_syncing = False

    def check_empty(self, folder_name):
        """
        return True if folder is empty
        """
        logging.debug("Checking if folder %s is empty" % folder_name)

        # leave old files
        if not self.app.delete_tmp_files:
            return True

        for r in range(0, self.nr_retries):
            self.app.attempt=r
            try:
                res = not os.listdir(folder_name)
            except OSError, e:
                logging.error("Failed to list dir: %s (%s)" % (folder_name, e))
                return False

            if res:
                return True

            logging.debug("Directory %s is not empty! Retrying [%d/%d] .." % (folder_name, r + 1, self.nr_retries))
            self.app.sync()
            #~ try:
                #~ shutil.rmtree(folder_name)
            #~ except OSError, e:
                #~ logging.error("Failed to delete folder: %s (%s)" % (folder_name, e))
                #~ return False

    @staticmethod
    def md5_for_file(fname, block_size=2**20):
        """
        calculates md5 of a file
        """
        fout = open(fname, 'r')
        md5 = hashlib.md5()
        while True:
            data = fout.read(block_size)
            if not data:
                break
            md5.update(data)
        fout.close()
        return md5.hexdigest()

    @staticmethod
    def touch(path):
        """
        create an empty file
        update utime
        """
        with open(path, 'a'):
            os.utime(path, None)

    @staticmethod
    def file_create(fname, fsize):
        """
        create a file of a size fsize and fill with a random data
        """
        fout = open(fname, 'wb')
        fout.write(get_random_str(fsize))
        fout.close()

    def files_create_size(self, first_symbol, maxsize, nr_files, dname, file_generate_name_func, l_files):
        """
        create a list of files of a specific size
        """
        for i in range(nr_files):
            fname = file_generate_name_func(first_symbol, i)
            ffname = os.path.join(dname, fname)
            if maxsize == 0:
                fsize = 0
            else:
                fsize = random.randint(1, maxsize)

            try:
                self.file_create(ffname, fsize)
            except IOError, e:
                logging.error("Failed to create file: %s (%s)" % (ffname, e))
                return False
            except UnicodeEncodeError, e:
                logging.debug("Discarded filename due to UnicodeEncodeError: %s" % (ffname))
                i=i-1
                continue
                
            md5_str = self.md5_for_file(ffname)
            l_files.append({"name":fname, "size":fsize, "md5":md5_str, "name_orig":fname})
            logging.debug("File created: %s [%s, %db]" % (ffname, md5_str, fsize))
        return True

    def files_create(self, file_generate_name_func=generate_ascii_name):
        """
        create files in "in" instance and check files presence in "out" instance
        Return list of files
        """
        logging.debug("Creating files.. (nrfiles="+str(self.nr_files)+")")

        l_files = []

        # empty files
        res = self.files_create_size("e", 0, self.nr_files, self.app.local_folder_in, file_generate_name_func, l_files)
        if not res:
            return None

        # small files < 1k
        if not hasattr(self.app, 'only_empty_files') or not self.app.only_empty_files:
            res = self.files_create_size("s", 1024, self.nr_files, self.app.local_folder_in, file_generate_name_func, l_files)
            if not res:
                return None
    
        if self.app.use_large_files:
            # medium files < 1mb
            res = self.files_create_size("m", 1024*1024, self.nr_files, self.app.local_folder_in, file_generate_name_func, l_files)
            if not res:
                return None

        # large files < 10mb
            res = self.files_create_size("l", 10*1024*1024, self.nr_files, self.app.local_folder_in, file_generate_name_func, l_files)
            if not res:
                return None

        # randomize list
        random.shuffle(l_files)

        return l_files

    def files_check(self, l_files, dir_name=""):
        """
        check files on both folders
        compare names, size, md5 sums
        """
        logging.debug("Checking files..")

        # check files
        for f in l_files:
            dd_out = os.path.join(self.app.local_folder_out, dir_name)
            ffname = os.path.join(dd_out, f["name"])
            #when saving mega alters some characters (we will look for 
            #destiny file having that in mind)
            ffname=normalizeandescape(ffname)
            
            
            dd_in = os.path.join(self.app.local_folder_in, dir_name)
            ffname_in = os.path.join(dd_in, f["name"])
            
            success = False

            logging.debug("Comparing %s and %s" % (ffname_in, ffname))
            # try to access the file
            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    with open(ffname):
                        pass
                    success = True
                    break
                except IOError as ex:
                    # wait for a file
                    logging.debug("File %s not found! Retrying [%d/%d] .." % (ffname, r + 1, self.nr_retries))
                    logging.debug("%s" % (ffname.encode("unicode-escape")))
                    self.app.sync()
            if success is False:
                logging.error("Failed to compare files: %s and %s" % (ffname_in, ffname))
                return False
            # get md5 of synced file
            md5_str = self.md5_for_file(ffname)
            if md5_str != f["md5"]:
                logging.error("MD5 sums don't match for file: %s" % ffname)
                return False
        return True

    def dir_create(self, dname, files_num, files_maxsize, file_generate_name_func=generate_ascii_name):
        """
        create and fill directory with files
        return files list
        """
        try:
            os.makedirs(dname)
        except OSError, e:
            logging.error("Failed to create directory: %s (%s)" % (dname, e))
            return None

        l_files = []
        res = self.files_create_size("s", files_maxsize, files_num, dname, file_generate_name_func, l_files)
        if not res:
            return None

        return l_files

    def dir_create_size(self, symbol, dirs_num, files_num, files_maxsize, parent_dir, dir_generate_name_func, l_dirs):
        """
        create dirs_num directories with  directories
        """
        for i in range(dirs_num):
            dname = dir_generate_name_func(symbol, i)
            ddname = os.path.join(parent_dir, dname)
            l_files = self.dir_create(ddname, files_num, files_maxsize, dir_generate_name_func)
            if l_files is None:
                logging.error("Failed to create directory: %s" % ddname)
                return False
            l_dirs.append({"name":dname, "files_nr":files_num, "name_orig":dname, "l_files":l_files})
            logging.debug("Directory created: %s [%d files]" % (ddname, files_num))
        return True

    def dirs_create(self, dir_generate_name_func=generate_ascii_name):
        """
        create dirs
        """
        logging.debug("Creating "+str(self.nr_dirs)+" directories..")

        l_dirs = []

        # create empty dirs
        res = self.dir_create_size("z", self.nr_dirs, 0, 0, self.app.local_folder_in, dir_generate_name_func, l_dirs)
        if not res:
            return None

        # create dirs with #nr_files files
        if not hasattr(self.app, 'only_empty_folders') or not self.app.only_empty_folders:
            res = self.dir_create_size("d", self.nr_dirs, self.nr_files, 1024, self.app.local_folder_in, dir_generate_name_func, l_dirs)
            if not res:
                return None

        # randomize list
        random.shuffle(l_dirs)

        return l_dirs

    def dirs_check(self, l_dirs):
        """
        check directories for both folders
        """
        logging.debug("Checking directories..")

        for d in l_dirs:
            dname = os.path.join(self.app.local_folder_out, d["name"])
            ##when saving mega alters some characters (we will look for 
            #destiny file having that in mind)
            dname=normalizeandescape(dname)
                
            dname_in = os.path.join(self.app.local_folder_in, d["name"])
            success = False

            logging.debug("Comparing dirs: %s and %s" % (dname_in, dname))

            # try to access the dir
            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    if os.path.isdir(dname):
                        success = True
                        break
                    else:
                        # wait for a dir
                        logging.debug("Directory %s not found! Retrying [%d/%d].." % (dname, r + 1, self.nr_retries))
                        self.app.sync()
                except OSError:
                    # wait for a dir
                    logging.debug("Directory %s not found! Retrying [%d/%d].." % (dname, r + 1, self.nr_retries))
                    self.app.sync()
            if success is False:
                logging.error("Failed to access directories: %s and " % dname)
                return False

            # check files
            res = self.files_check(d["l_files"], d["name"])
            if not res:
                logging.error("Directories do not match !")
                return False

        return True

    def file_rename(self, ffname_src, ffname_dst):
        """
        renaming file
        return True if renamed
        """
        for r in range(0, self.nr_retries):
            self.app.attempt=r
            if os.path.exists(ffname_src):
                try:
                    shutil.move(ffname_src, ffname_dst)
                except OSError, e:
                    logging.error("Failed to rename file: %s (%s)" % (ffname_src, e))
                    return False

            if self.force_syncing:
                self.app.sync()

            # try to access both files (old and new)
            if not os.path.exists(ffname_dst):
                logging.debug("Failed to access a newly renamed file: %s, retrying [%d/%d].." % (ffname_dst, r + 1, self.nr_retries))
                continue
            if os.path.exists(ffname_src):
                logging.debug("Still can access an old renamed file: %s, retrying [%d/%d]" % (ffname_src, r + 1, self.nr_retries))
                continue
            break

        # try to access both files (old and new)
        if not os.path.exists(ffname_dst):
            logging.error("Failed to access a newly renamed file: %s. Aborting.." % ffname_dst)
            return False
        if os.path.exists(ffname_src):
            logging.error("Still can access an old renamed file: %s. Aborting.." % ffname_src)
            return False
        return True

    def files_rename(self, l_files, file_generate_name_func=generate_ascii_name):
        """
        rename objects in "in" instance and check new files in "out" instance
        """
        logging.debug("Renaming files..")

        i = 0
        for f in l_files:
            ffname_src = os.path.join(self.app.local_folder_in, f["name"])
            f["name"] = file_generate_name_func("renamed_", i)
            i = i + 1
            ffname_dst = os.path.join(self.app.local_folder_in, f["name"])

            logging.debug("Renaming file: %s => %s" % (ffname_src, ffname_dst))

            if not self.file_rename(ffname_src, ffname_dst):
                return False

        return True
        
    def files_moveanddelete(self, l_files, where=".", timeout=0, file_generate_name_func=generate_ascii_name):
        """
        moves and deletes objects in "in" instance and check new files in "out" instance
        """
        logging.debug("Move&Rename files..")
        try:
            os.makedirs(os.path.join(self.app.local_folder_in,where))
        except Exception, e:
            logging.debug("Unable to create subfolder: %s (%s)" % (where, e))

        i = 0
        for f in l_files:
            ffname_src = os.path.join(self.app.local_folder_in, f["name"])
            f["name"] = file_generate_name_func("renamed_", i)
            i = i + 1
            ffname_dst = os.path.join(self.app.local_folder_in, where, f["name"])

            logging.debug("move&delete file: %s => %s" % (ffname_src, ffname_dst))

            if os.path.exists(ffname_src):
                try:
                    shutil.move(ffname_src, ffname_dst)
                except OSError, e:
                    logging.error("Failed to rename file: %s (%s)" % (ffname_src, e))
                    return False
            try:
                time.sleep(timeout)
                os.remove(ffname_dst)
            except OSError, e:
                logging.error("Failed to delete file: %s (%s)" % (ffname_dst, e))
                return False

        if (where != "."):
            shutil.rmtree(os.path.join(self.app.local_folder_in,where))
        return True
        
    def files_mimic_update_with_backup(self, l_files, timeout=0, file_generate_name_func=generate_ascii_name):
        """
        moves and deletes objects in "in" instance and check new files in "out" instance
        """
        logging.debug("Mimic update with backup files..")

        i = 0
        for f in l_files:
            ffname_src = os.path.join(self.app.local_folder_in, f["name"])
            #f["name"] = file_generate_name_func("renamed_", i)
            i = i + 1
            ffname_dst = os.path.join(self.app.local_folder_in, "renamed_"+f["name"])
            ffname_dst_out = os.path.join(self.app.local_folder_out, "renamed_"+f["name"])

            logging.debug("Mimic update with backup file: %s => %s" % (ffname_src, ffname_dst))

            if os.path.exists(ffname_src):
                try:
                    shutil.move(ffname_src, ffname_dst)
                except OSError, e:
                    logging.error("Failed to rename file: %s (%s)" % (ffname_src, e))
                    return False
            try:
                time.sleep(timeout)
                with open(ffname_dst, 'r') as f:
                    with open(ffname_src, 'w') as f2:
                        for r in range(100):
                            f2.write("whatever")
                            time.sleep(0.03)
                            if os.path.exists(ffname_dst_out): #existing temporary file
                                logging.error("ERROR in sync: Temporary file being created in syncout: : %s!" % (ffname_dst))
                                os.remove(ffname_dst)
                                return False;
                os.remove(ffname_dst)
            except OSError, e:
                logging.error("Failed to delete file: %s (%s)" % (ffname_dst, e))
                return False

        return True

    def files_remove(self, l_files):
        """
        remove files in "in" instance and check files absence in "out" instance
        """
        logging.debug("Removing files..")

        for f in l_files:
            ffname = os.path.join(self.app.local_folder_in, f["name"])

            logging.debug("Deleting: %s" % ffname)

            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    os.remove(ffname)
                except OSError, e:
                    logging.error("Failed to delete file: %s (%s)" % (ffname, e))
                    return False

                if self.force_syncing:
                    self.app.sync()

                # check if local file does not exist
                if not os.path.exists(ffname):
                    break
                logging.debug("Deleted file %s still exists, retrying [%d/%d].." % (ffname, r + 1, self.nr_retries))

            if os.path.exists(ffname):
                logging.debug("Deleted file %s still exists, aborting.." % ffname)

        success = False
        for f in l_files:
            ffname = os.path.join(self.app.local_folder_out, f["name"])
            
            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    # file must be deleted
                    with open(ffname):
                        pass
                    logging.debug("File %s is not deleted. Retrying [%d/%d] .." % (ffname, r + 1, self.nr_retries))
                    self.app.sync()
                except IOError:
                    success = True
                    break
            if success is False:
                logging.error("Failed to delete file: %s" % ffname)
                return False
        return True

    def dirs_rename(self, l_dirs, dir_generate_name_func=generate_ascii_name):
        """
        rename directories in "in" instance and check directories new names in "out" instance
        """
        logging.debug("Renaming directories..")

        i = 0
        for d in l_dirs:
            dname_src = os.path.join(self.app.local_folder_in, d["name"])
            d["name"] = dir_generate_name_func("renamed_", i)
            i = i + 1
            dname_dst = os.path.join(self.app.local_folder_in, d["name"])
            try:
                shutil.move(dname_src, dname_dst)
            except OSError, e:
                logging.error("Failed to rename directory: %s (%s)" % (dname_src, e))
                return False

            if self.force_syncing:
                self.app.sync()

            # try to both dirs
            if not os.path.exists(dname_dst):
                logging.error("Failed to access a newly renamed directory: %s" % dname_dst)
                return False
            if os.path.exists(dname_src):
                logging.error("Still can access an old directory: %s" % dname_src)
                return False

            logging.debug("Directory renamed: %s => %s" % (dname_src, dname_dst))

        return True

    def dirs_remove(self, l_dirs):
        """
        remove directories in "in" instance and check directories absence in "out" instance
        """
        logging.debug("Removing directories..")

        for d in l_dirs:
            dname = os.path.join(self.app.local_folder_in, d["name"])
            try:
                shutil.rmtree(dname)
            except OSError, e:
                logging.error("Failed to delete dir: %s (%s)" % (dname, e))
                return False
            logging.debug("Directory removed: %s" % dname)

            if self.force_syncing:
                self.app.sync()

            if os.path.exists(dname):
                logging.error("Still can access a renamed directory: %s" % dname)
                return False

        success = False
        for d in l_dirs:
            dname = os.path.join(self.app.local_folder_out, d["name"])
            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    # dir must be deleted
                    if not os.path.isdir(dname):
                        success = True
                        break
                    logging.debug("Directory %s is not deleted! Retrying [%d/%d] .." % (dname, r + 1, self.nr_retries))
                    self.app.sync()
                except OSError:
                    success = True
                    break
            if success is False:
                logging.error("Failed to delete dir: %s" % dname)
                return False
        return True

    def dirs_check_empty(self):
        """
        return True if both folders are empty
        """
        return self.check_empty(self.app.local_folder_in) and self.check_empty(self.app.local_folder_out)

    def local_tree_create_dir(self, parent_dir):
        """
        generate directory
        """
        strlen = random.randint(10, 20)
        dname = get_random_str(size=strlen)
        ddname = os.path.join(parent_dir, dname)
        real_dname = os.path.join(self.app.local_folder_in, ddname)

        try:
            os.makedirs(real_dname)
        except OSError, e:
            logging.error("Failed to create directory: %s (%s)" % (real_dname, e))
            return None, None, None, None

        # populate with random amount of files
        obj_nr = random.randint(1, self.local_obj_nr)
        l_files = []
        for _ in range(0, obj_nr):
            strlen = random.randint(10, 20)
            fname = get_random_str(size=strlen) + ".txt"
            ffname = os.path.join(ddname, fname)
            fname_real = os.path.join(self.app.local_folder_in, ffname)
            try:
                self.file_create(fname_real, random.randint(10, 100))
            except IOError, e:
                logging.error("Failed to create file: %s (%s)" % (fname_real, e))
                return None, None, None, None
            l_files.append({"name":fname, "fname":ffname})

        # populate with random amount of dirs
        obj_nr = random.randint(1, self.local_obj_nr)
        l_dirs = []
        for _ in range(0, obj_nr):
            strlen = random.randint(10, 20)
            cname = get_random_str(size=strlen)
            ccname = os.path.join(ddname, cname)
            cname_real = os.path.join(self.app.local_folder_in, ccname)
            try:
                os.makedirs(cname_real)
            except OSError, e:
                logging.error("Failed to create directory: %s (%s)" % (cname_real, e))
                return None, None, None, None
            l_dirs.append({"name":cname, "fname":ccname})

            # recursively create subtree
        return dname, ddname, l_files, l_dirs

    def local_tree_create(self, parent_dir, dirs_nr):
        """
        generate local directory tree, recursively populate with random number of directories / files
        return list of dictionaries
        """
        if dirs_nr == 0:
            return None

        l_tree = []

        for _ in range(0, dirs_nr):
            dname, ddname, l_files, l_dirs = self.local_tree_create_dir(parent_dir)
            if dname is None:
                return None
            l_child = self.local_tree_create(ddname, dirs_nr - 1)
            l_tree.append({"name":dname, "fname":ddname, "files":l_files, "dirs":l_dirs, "child":l_child})
        return l_tree

    def local_tree_get_dir(self, l_tree):
        """
        directory walk generator
        Returns relative directory path
        """
        for i in l_tree:
            yield i["fname"]
            for dd in i["dirs"]:
                yield dd["fname"]
            if i["child"] is not None:
                for x in self.local_tree_get_dir(i["child"]):
                    yield x

    def local_tree_get_dirs(self, l_tree):
        """
        directory walk generator
        Returns dir dict
        """
        for i in l_tree:
            yield i
            if i["child"] is not None:
                for x in self.local_tree_get_dirs(i["child"]):
                    yield x

    def local_tree_get_file(self, l_tree):
        """
        directory walk generator
        Returns relative file path
        """
        for i in l_tree:
            for ff in i["files"]:
                yield ff["fname"]
            if i["child"] is not None:
                for x in self.local_tree_get_file(i["child"]):
                    yield x

    def local_tree_compare(self, l_tree):
        """
        compare two local trees
        return True if they are the same
        """
        total_dirs = total_files = 0

        # try to access directories in "out" folder
        for d in self.local_tree_get_dir(l_tree):
            dname = os.path.join(self.app.local_folder_out, d)
            success = False
            total_dirs = total_dirs + 1

            # logging.debug("Trying to access dir: %s" % dname)
            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    if os.path.isdir(dname):
                        success = True
                        break
                    else:
                        # wait for a dir
                        logging.debug("Directory %s not found! Retrying [%d/%d].." % (dname, r + 1, self.nr_retries))
                        self.app.sync()
                except OSError:
                    # wait for a dir
                    logging.debug("Directory %s not found! Retrying [%d/%d].." % (dname, r + 1, self.nr_retries))
                    self.app.sync()
            if success is False:
                logging.error("Failed to access directories: %s and " % dname)
                return False

        # try to access files in "out" folder
        for f in self.local_tree_get_file(l_tree):
            fname = os.path.join(self.app.local_folder_out, f)
            success = False
            total_files = total_files + 1

            # logging.debug("Trying to access file: %s" % fname)
            for r in range(0, self.nr_retries):
                self.app.attempt=r
                try:
                    with open(fname):
                        pass
                    success = True
                    break
                except IOError:
                    # wait for a file
                    logging.debug("File %s not found! Retrying [%d/%d] .." % (fname, r + 1, self.nr_retries))
                    self.app.sync()
            if success is False:
                logging.error("Failed to access file: %s" % fname)
                return False

        logging.debug("Total dirs: %d, files: %d" % (total_dirs, total_files))
        return True

    def local_tree_create_and_move(self, l_tree):
        """
        create a folder and fill with content
        randomly select an existing folder and create a subfolder of it
        move the first folder to the newly created one
        compare results, repeat 10 times
        return True if success
        """

        for _ in range(0, self.nr_dirs):
            # Create a dir
            l_dir = []
            dname, ddname, l_files, l_dirs = self.local_tree_create_dir("")
            if dname is None:
                return False
            logging.debug("Directory created: %s" % ddname)
            l_dir.append({"name":dname, "fname":ddname, "files":l_files, "dirs":l_dirs, "child":None})

            # wait for a sync and compare
            if not self.local_tree_compare(l_dir):
                return False

            # select random existing folder
            dir_dicts_l = [d for d in self.local_tree_get_dirs(l_tree)]
            dd = random.choice(dir_dicts_l)

            if dd["dirs"] is None:
                dd["dirs"] = []

            # create a new subfolder
            strlen = random.randint(10, 20)
            dname = get_random_str(size=strlen)
            ddname = os.path.join(dd["fname"], dname)
            dname_real = os.path.join(self.app.local_folder_in, ddname)
            logging.debug("Creating new dir: %s, parent: %s" % (ddname, dd["fname"]))
            try:
                os.makedirs(dname_real)
            except OSError, e:
                logging.error("Failed to create directory: %s (%s)" % (dname_real, e))
                return False

            dd["dirs"].append({"name":dname, "fname":ddname, "files":None, "dirs":None, "child":None})

            # move existing folder into newly created folder
            old_name = os.path.join(self.app.local_folder_in, l_dir[0]["fname"])
            new_name = os.path.join(dname_real, l_dir[0]["name"])

            logging.debug("Moving %s to %s" % (old_name, new_name))

            try:
                shutil.move(old_name, new_name)
            except OSError, e:
                logging.error("Failed to move dir: %s to new: %s (%s)" % (old_name, new_name, e))
                return False

            # fix existing dir dict
            new_ffname = os.path.join(ddname, l_dir[0]["name"])
            l_dir[0]["fname"] = new_ffname
            for f in l_dir[0]["files"]:
                f["ffname"] = os.path.join(new_ffname, f["name"])
            for d in l_dir[0]["dirs"]:
                d["ffname"] = os.path.join(new_ffname, d["name"])

            # wait for a sync and compare
            if not self.local_tree_compare(l_tree):
                return False

        # all good !
        return True

    def local_tree_multiple_renames(self, l_tree):
        """
        perform several object renames
        then rename back to the original name
        """

        # rename dirs
        for _ in range(0, self.nr_changes):
            # select random existing folder
            dir_dicts_l = [d for d in self.local_tree_get_dirs(l_tree)]
            dd = random.choice(dir_dicts_l)
            if dd["dirs"] is None:
                continue

            oname = dd["dirs"][0]["name"]
            odname = os.path.join(dd["fname"], oname)
            orig_name = os.path.join(self.app.local_folder_in, odname)
            prev_name = orig_name

            # rename 10 times
            for _ in range(0, self.nr_changes):
                strlen = random.randint(10, 20)
                dname = get_random_str(size=strlen)
                ddname = os.path.join(dd["fname"], dname)
                dname_real = os.path.join(self.app.local_folder_in, ddname)

                logging.debug("Renaming %s to %s" % (prev_name, dname_real))

                if not self.file_rename(prev_name, dname_real):
                    return False

                prev_name = dname_real

            # rename back to origin
            logging.debug("Moving %s to %s" % (prev_name, orig_name))

            if not self.file_rename(prev_name, orig_name):
                return False

            self.app.sync()

            # wait for a sync and compare
            if not self.local_tree_compare(l_tree):
                return False

        # rename files
        for _ in range(0, self.nr_changes):
            # select random existing folder
            dir_dicts_l = [d for d in self.local_tree_get_dirs(l_tree)]
            dd = random.choice(dir_dicts_l)
            if dd["files"] is None:
                continue

            oname = dd["files"][0]["name"]
            odname = os.path.join(dd["fname"], oname)
            orig_name = os.path.join(self.app.local_folder_in, odname)
            prev_name = orig_name

            # rename 10 times
            for _ in range(0, self.nr_changes):
                strlen = random.randint(10, 20)
                dname = get_random_str(size=strlen)
                ddname = os.path.join(dd["fname"], dname)
                dname_real = os.path.join(self.app.local_folder_in, ddname)

                logging.debug("Renaming %s to %s" % (prev_name, dname_real))

                if not self.file_rename(prev_name, dname_real):
                    return False

                prev_name = dname_real

            # rename back to origin
            logging.debug("Moving %s to %s" % (prev_name, orig_name))

            if not self.file_rename(prev_name, orig_name):
                return False

            self.app.sync()

            # wait for a sync and compare
            if not self.local_tree_compare(l_tree):
                return False

        return True



"""
 Base class for testing syncing algorithm

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

# TODO tests:
# * "pause" sync
# * lock directory
# * large (> 4Gb) files
# * > 10000 folders to synchronize

from sync_test_base import SyncTestBase
from sync_test_base import get_random_str
from sync_test_base import generate_unicode_name
import random
import os
import logging
import time
import math

class SyncTest(SyncTestBase):
    """
    Class with MEGA SDK test methods
    """

# tests
    def test_create_delete_files(self):
        """
        create files with different size,
        compare files on both folders,
        remove files, check that files removed from the second folder
        """
        logging.info("Launching test_create_delete_files test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create files
        l_files = self.files_create()
        self.assertIsNotNone(l_files, "Creating files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # remove files
        self.assertTrue(self.files_remove(l_files), "Removing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True

    def test_create_rename_delete_files(self):
        """
        create files with different size,
        compare files on both folders,
        rename files
        """
        logging.info("Launching test_create_rename_delete_files test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();        

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create files
        l_files = self.files_create()
        self.assertIsNotNone(l_files, "Creating files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # renaming
        self.assertTrue(self.files_rename(l_files), "Renaming files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # remove files
        self.assertTrue(self.files_remove(l_files), "Removing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True
        
    def test_create_move_delete_files(self):
        """
        create files with different size,
        move and delete files
        """
        logging.info("Launching test_create_move_delete_files test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();        

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create files
        l_files = self.files_create()
        self.assertIsNotNone(l_files, "Creating files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # move & delete
        self.assertTrue(self.files_moveanddelete(l_files,"subfolder"), "Move&Delete files")
        #~ self.assertTrue(self.files_moveanddelete(l_files,"."), "Move&Delete files") #this is expected to fail as of 20171019        
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True
        
    def test_mimic_update_with_backup_files(self):
        """
        create files with different size,
        mimic_update_with_backup
        """
        logging.info("Launching test_mimic_update_with_backup_files test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();        

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create files
        l_files = self.files_create()
        self.assertIsNotNone(l_files, "Creating files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # mimic update with backup
        self.assertTrue(self.files_mimic_update_with_backup(l_files), "Mimic update with backup files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # remove files
        self.assertTrue(self.files_remove(l_files), "Removing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True

    def test_create_delete_dirs(self):
        """
        create directories with different amount of files,
        compare directories on both sync folders,
        remove directories, check that directories removed from the second folder
        """
        logging.info("Launching test_create_delete_dirs test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();
        

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create dirs
        l_dirs = self.dirs_create()
        self.assertIsNotNone(l_dirs, "Creating directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.dirs_check(l_dirs), "Comparing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # remove files
        self.assertTrue(self.dirs_remove(l_dirs), "Removing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True

    def test_create_rename_delete_dirs(self):
        """
        create directories with different amount of files,
        compare directories on both sync folders,
        rename directories
        compare directories on both sync folders,
        remove directories, check that directories removed from the second folder
        """
        logging.info("Launching test_create_rename_delete_dirs test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.change_folders();

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create dirs
        l_dirs = self.dirs_create()
        self.assertIsNotNone(l_dirs, "Creating directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.dirs_check(l_dirs), "Comparing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # rename dirs
        self.assertTrue(self.dirs_rename(l_dirs), "Rename directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.dirs_check(l_dirs), "Comparing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # remove files
        self.assertTrue(self.dirs_remove(l_dirs), "Removing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True

    def test_sync_files_write(self):
        """
        write data to a file located in both sync folders
        check for the result, expected result: both files contains the same content
        """

        logging.info("Launching test_sync_files_write test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();

        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        for _ in range(0, self.nr_files):
            self.assertTrue(self.app.is_alive(), "Test application is not running")
            strlen = random.randint(10, 20)
            fname = get_random_str(size=strlen)

            fname_in = os.path.join(self.app.local_folder_in, fname)
            fname_out = os.path.join(self.app.local_folder_out, fname)

            logging.debug("Writing to both files: %s and %s" % (fname_in, fname_out))

            with open(fname_in, 'a'):
                os.utime(fname_in, None)
            with open(fname_out, 'a'):
                os.utime(fname_out, None)

            #self.app.sync()

            for _ in range(self.nr_changes):
                with open(fname_in, 'a') as f_in:
                    f_in.write(get_random_str(100))

                with open(fname_out, 'a') as f_out:
                    f_out.write(get_random_str(100))

                for r in range(self.app.nr_retries):
                    self.app.attempt=r
                    md5_in = "INPUT FILE NOT READABLE";
                    md5_out = "OUTPUT FILE NOT READABLE";

                    try:
                        md5_in = self.md5_for_file(fname_in)
                        md5_out = self.md5_for_file(fname_out)
                    except IOError:
                        pass;

                    if md5_in == md5_out:
                        break
                    self.app.sync()

                logging.debug("File %s md5: %s" % (fname_in, md5_in))
                logging.debug("File %s md5: %s" % (fname_out, md5_out))

                self.assertEqual(md5_in, md5_out, "Files do not match")


    def test_local_operations(self):
        """
        write data to a file located in both sync folders
        check for the result, expected result: both files contains the same content
        """
        logging.info("Launching test_local_operations test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.change_folders();
        
        l_tree = self.local_tree_create("", self.nr_dirs)
        self.assertIsNotNone(l_tree, "Failed to create directory tree!")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.app.sync()
        self.assertTrue(self.local_tree_compare(l_tree), "Failed to compare directory trees!")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.assertTrue(self.local_tree_create_and_move(l_tree), "Failed to create a new sub folder and move an existing directory into it!")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        self.assertTrue(self.local_tree_multiple_renames(l_tree), "Failed to rename folder multiple times and then rename back to the original name!")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

    def test_update_mtime(self):
        """
        update mtime of a file in both local folders
        """
        logging.info("Launching test_update_mtime test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        
        self.app.change_folders();

        in_file = os.path.join(self.app.local_folder_in, "mtime_test")
        out_file = os.path.join(self.app.local_folder_out, "mtime_test")

        for _ in range(self.nr_time_changes):
            logging.debug("Touching: %s" % in_file)
            now = math.floor(time.time()) #floor to get seconds
            with open(in_file, 'a'):
                os.utime(in_file, (now, now))

      #      with open(out_file, 'a'):
      #          os.utime(in_file, (now, now))
            
            atime=0
            mtime=0
            for r in range(self.app.nr_retries):
                self.app.attempt=r
                try:
                    mtime = os.path.getmtime(out_file)
                except OSError:
                    pass

                try:
                    atime = os.path.getatime(out_file)
                except OSError:
                    pass
            
                logging.debug("Comparing time: %s. atime: %d = %d, mtime: %d = %d" % (out_file, now, atime, now, mtime))
                
                if (mtime==now): #all good
                    break;
                self.app.sync()
                logging.debug("Comparing time for %s failed! Retrying [%d/%d] .." % (out_file, r + 1, self.nr_retries))

            
            #self.assertEqual(atime, now, "atime values are different")
            self.assertEqual(mtime, now, "mtime values are different")
            
            self.assertTrue(self.app.is_alive(), "Test application is not running")

    def test_create_rename_delete_unicode_files_dirs(self):
        """
        create directories with different amount of files,
        using Unicode encoding for files / directories names,
        compare directories on both sync folders,
        rename directories
        compare directories on both sync folders,
        remove directories, check that directories removed from the second folder
        """
        logging.info("Launching test_create_rename_delete_unicode_files_dirs test")
        self.assertTrue(self.app.is_alive(), "Test application is not running")
        
        self.app.change_folders();
        
        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create files
        l_files = self.files_create(generate_unicode_name)
        self.assertIsNotNone(l_files, "Creating files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # renaming
        self.assertTrue(self.files_rename(l_files, generate_unicode_name), "Renaming files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.files_check(l_files), "Comparing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # remove files
        self.assertTrue(self.files_remove(l_files), "Removing files")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty #TODO: why is this twice?
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # create dirs
        l_dirs = self.dirs_create(generate_unicode_name)
        self.assertIsNotNone(l_dirs, "Creating directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.dirs_check(l_dirs), "Comparing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # rename dirs
        self.assertTrue(self.dirs_rename(l_dirs, generate_unicode_name), "Rename directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        self.app.sync()

        # comparing
        self.assertTrue(self.dirs_check(l_dirs), "Comparing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # remove files
        self.assertTrue(self.dirs_remove(l_dirs), "Removing directories")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        # make sure remote folders are empty
        self.assertTrue(self.dirs_check_empty(), "Checking if remote folders are empty")
        self.assertTrue(self.app.is_alive(), "Test application is not running")

        return True



#!/usr/bin/python

# This python script can be used to extract HTTP request and responses from an SDK log
# and beautify json payloads

from __future__ import print_function
import json, sys, re
import argparse

parser = argparse.ArgumentParser()

parser.add_argument('--print-cs-post', '-r', action='store_true', dest='postcs', help='print lines with cs POST')
parser.set_defaults(postcs=False)

parser.add_argument('--print-sc-post', '-a', action='store_true', dest='postsc', help='print lines with sc POST')
parser.set_defaults(postsc=False)

parser.add_argument('--only-action-packets', '-s', action='store_true', dest='onlyactionpackets', help='do not print cs requests/responses')
parser.set_defaults(onlyactionpackets=False)

parser.add_argument('--only-client-requests', '-c', action='store_true', dest='onlyclientreqs', help='do not print sc requests/responses')
parser.set_defaults(onlyclientreqs=False)

parser.add_argument('--include-lines-matching', '-i', dest='includepattern', help='include lines matching some pattern')
parser.set_defaults(includepattern=None)


parser.add_argument('file', nargs=argparse.REMAINDER)

args = parser.parse_args()
fToParse = sys.stdin if not len(args.file) else open(args.file[0])

scpatterns=["sc Received", "sc Sending"]
cspatterns=["cs Received", "cs Sending"]
patterns=[]

if not args.onlyactionpackets:
    patterns+=(cspatterns)
if not args.onlyclientreqs:
    patterns+=(scpatterns)


for l in fToParse:

    if args.postcs and "cs POST target" in l:
        print (l.strip(),)
    if args.postsc and "sc POST target" in l:
        print (l.strip(),)
    if any(x in l for x in patterns) and "sc Received 1: 0" not in l and " sc Sending 0:" not in l:

        m = re.search('(.*): (\{.*\}|\[.*\])', l)
        if m:
            header = found = m.group(1)
            found = m.group(2)
            if header and found:
                try:
                    contents = json.dumps(json.loads(found), sort_keys=False, indent=4)
                    print (header+contents)
                except:
                    #try this other format:
                    m = re.search('(.*)((sc|cs) (Received|Sending) [0-9]*:  *)(\{.*\}|\[.*\])  *(\[.*cpp.*\])*', l)
                    if m and m is not None and len(m.groups()) > 4:
                        header = found = m.group(1)
                        sendrecv = m.group(2)
                        found = m.group(5)
                        fil = m.group(6)
                        if header and sendrecv and found:
                            try:
                                contents = json.dumps(json.loads(found), sort_keys=False, indent=4)
                                print (header+sendrecv+contents)
                            except:
                                print (l)
                        else:
                            print (l,)
                    else:
                        print (l,)
            else:
                print (l,)
        else:
            print (l,)
    elif args.includepattern:
        m = re.search(args.includepattern, l)
        if m:
            print (l.strip(),)



"""
 Application for testing syncing algorithm

 (c) 2013-2014 by Mega Limited, Wellsford, New Zealand

 This file is part of the MEGA SDK - Client Access Engine.

 Applications using the MEGA API must present a valid application key
 and comply with the the rules set forth in the Terms of Service.

 The MEGA SDK is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 @copyright Simplified (2-clause) BSD License.

 You should have received a copy of the license along with this
 program.
"""

import os
import time
import random
from sync_test_base import get_random_str
import shutil
import logging
import datetime


class SyncTestApp(object):
    """
    test application base class
    """
    def __init__(self, local_mount_in, local_mount_out, work_folder, delete_tmp_files=True, use_large_files=True):
        """
        work_dir: a temporary folder to place generated files
        remote_folder: a remote folder to sync
        """

        self.start_time = time.time()

        random.seed(time.time())

        self.local_mount_in = local_mount_in
        self.local_mount_out = local_mount_out

        self.rnd_folder = get_random_str()
        self.local_folder_in = os.path.join(self.local_mount_in, self.rnd_folder)
        self.local_folder_out = os.path.join(self.local_mount_out, self.rnd_folder)
        self.work_folder = os.path.join(work_folder, self.rnd_folder)

        self.nr_retries = 200
        self.delete_tmp_files = delete_tmp_files
        self.use_large_files = use_large_files
    
    def change_folders(self):
        """
        cleans directories and call finish
        """
        time.sleep(0.2) # to prevent from sync algorithm interpreting we are renaming
        
        if self.delete_tmp_files:
            try:
                shutil.rmtree(self.local_folder_in)
            except OSError:
                pass
        time.sleep(1.2) # to prevent from sync algorithm interpreting we are renaming
        self.rnd_folder = get_random_str()
        self.local_folder_in = os.path.join(self.local_mount_in, self.rnd_folder)
        self.local_folder_out = os.path.join(self.local_mount_out, self.rnd_folder)
        self.work_folder = os.path.join(self.work_folder, self.rnd_folder) 
        
        self.prepare_folders();          

    def __enter__(self):
        # call subclass function
        res = self.start()
        if not res:
            self.stop()
            raise Exception('Failed to start app!')

        res = self.prepare_folders()
        if not res:
            self.stop()
            raise Exception('Failed to start app!')

        return self

    def __exit__(self, exc_type, exc_value, traceback):
        # remove tmp folders
        if self.delete_tmp_files:
            try:
                logging.debug("Deleting %s" % self.local_folder_in)
                shutil.rmtree(self.local_folder_in)
            except OSError:
                pass
            try:
                logging.debug("Deleting %s" % self.local_folder_out)
                shutil.rmtree(self.local_folder_out)
            except OSError:
                pass
            try:
                logging.debug("Deleting %s" % self.work_folder)
                shutil.rmtree(self.work_folder)
            except OSError:
                pass

        # terminate apps
        self.stop()
        logging.info("Execution time: %s" % str(datetime.timedelta(seconds=time.time()-self.start_time)))

    @staticmethod
    def touch(path):
        """
        create an empty file
        update utime
        """
        with open(path, 'a'):
            os.utime(path, None)

    def prepare_folders(self):
        """
        prepare upsync, downsync and work directories
        """
        # create "in" folder
        logging.info("IN folder: %s" % self.local_folder_in)
        try:
            os.makedirs(self.local_folder_in)
        except OSError, e:
            logging.error("Failed to create directory: %s (%s)" % (self.local_folder_in, e))
            return False

        logging.info("OUT folder: %s" % self.local_folder_out)

        self.sync()

        # temporary workaround
        #tmp_fix_file = os.path.join(self.local_mount_out, "tmp_fix")

        success = False
        # try to access the dir
        for r in range(0, self.nr_retries):
            self.attempt=r
            try:
                if os.path.isdir(self.local_folder_out):
                    success = True
                    break
                else:
                    # wait for a dir
                    logging.debug("Directory %s not found! Retrying [%d/%d] .." % (self.local_folder_out, r + 1, self.nr_retries))
                    #self.touch(tmp_fix_file)
                    self.sync()
            except OSError:
                # wait for a dir
                logging.debug("Directory %s not found! Retrying [%d/%d] .." % (self.local_folder_out, r + 1, self.nr_retries))
                #self.touch(tmp_fix_file)
                self.sync()
        if success is False:
            logging.error("Failed to access directory: %s" % self.local_folder_out)
            return False

        # create work folder
        logging.debug("Work folder: %s" % self.work_folder)
        try:
            os.makedirs(self.work_folder)
        except OSError, e:
            logging.error("Failed to create directory: %s (%s)" % (self.work_folder, e))
            return False

        return True

    def stop(self):
        """
        cleans directories and call finish
        """
        if self.delete_tmp_files:
            try:
                shutil.rmtree(self.local_folder_in)
            except OSError:
                pass
        self.sync()
        self.finish()

# virtual methods
    def start(self):
        """
        start application
        """
        raise NotImplementedError("Not Implemented !")

    def finish(self):
        """
        stop application
        """
        raise NotImplementedError("Not Implemented !")

    def sync(self):
        """
        wait for full synchronization
        """
        raise NotImplementedError("Not Implemented !")

    def pause(self):
        """
        pause application
        """
        raise NotImplementedError("Not Implemented !")

    def unpause(self):
        """
        unpause application
        """
        raise NotImplementedError("Not Implemented !")

    def is_alive(self):
        """
        return True if application instance is running
        """
        raise NotImplementedError("Not Implemented !")



#!/bin/env python
# -*- coding: utf-8 -*-
"""
This setup is loosely following the instructions adapted from
https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/
"""
import os
import re

from setuptools import setup


def read(fname):
    """
    Utility function to read the README file.
    Used for the long_description.  It's nice, because now
    1) we have a top level README file and
    2) it's easier to type in the README file than to put a raw string in below ...
    """
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def get_version():
    """
    Grabs and returns the version and release numbers from autotools.
    """
    version_h = open(os.path.join('..', '..', 'include', 'mega', 'version.h')).read()
    major = re.search('define MEGA_MAJOR_VERSION ([0-9]+)',
                      version_h)
    minor = re.search('define MEGA_MINOR_VERSION ([0-9]+)',
                      version_h)
    micro = re.search('define MEGA_MICRO_VERSION (.+?)',
                      version_h)
    if major:
        major, minor, micro = major.group(1), minor.group(1), micro.group(1)
        version = '.'.join([major, minor])
    else:
        version = 'raw_development'
    if micro:
        release = '.'.join([major, minor, micro])
    else:
        release = 'raw_development'
    return version, release

def make_symlink(src, dst):
    """Makes a symlink, ignores errors if it's there already."""
    try:
        os.symlink(src, dst)
    except OSError as e:
        if e.strerror != 'File exists':
            raise e

def remove_file(fname):
    """Removes a file/link, ignores errors if it's not there any more."""
    try:
        os.remove(fname)
    except OSError as e:
        if e.strerror != 'No such file or directory':
            raise e

# Put native library modules into a "good place" for the package.
make_symlink('../../src/.libs/libmega.so', 'libmega.so')
make_symlink('.libs/_mega.so', '_mega.so')

# Create a dummy __init__.py if not present.
_init_file = '__init__.py'
_init_file_created = False
if not os.path.exists(_init_file):
    with open(_init_file, 'wb') as fd:
        _init_file_created = True

setup(
    name='megasdk',
    version=get_version()[1],
    description='Python bindings to the Mega file storage SDK.',
    long_description=read('DESCRIPTION.rst'),
    url='http://github.com/meganz/sdk/',
    license='Simplified BSD',
    author='Guy Kloss',
    author_email='gk@mega.co.nz',
    packages=['mega'],
    package_dir={'mega': '.'},
    package_data = {
        'mega': ['libmega.so', '_mega.so'],
    },
    exclude_package_data = {'': ['test_libmega.py']},
    include_package_data=True,
    keywords=['MEGA', 'privacy', 'cloud', 'storage', 'API'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)

# Clean up some temporary stuff.
remove_file('libmega.so')
remove_file('_mega.so')
if _init_file_created:
    remove_file(_init_file)
